rm(list=ls())
#setwd(path)
setwd(dirname(rstudioapi::getActiveDocumentContext()$path))
# install and load the relevant packages 
# packages used
listofpackages <- c("dygraphs", "dplyr","ellipse","reshape2","ggplot2","xts","readxl","xlsx","stats",
                    "plotly", "hrbrthemes")

for (j in listofpackages){
  if(sum(installed.packages()[, 1] == j) == 0) {
    install.packages(j)
  }
  library(j, character.only = T)
}


### Data loading, note that the MSList.csv allows to choose countries included in the analysis form countries left as residuals
MSMonthlyRating <- read.table("data/MSMonthlyRating.csv",header = T,sep = ";",dec = ".", stringsAsFactors = FALSE)           ### MS Monthly Rating  
MSMonthlyDebt <- read.table("data/MSMonthlyDebt.csv",header = T,sep = ";",dec = ".", stringsAsFactors = FALSE)               ### MS Monthly Debt Outstanding
MS10YY <- read.table("data/MS10YY.csv",header = T,sep = ";",dec = ".", stringsAsFactors = FALSE)     ### 10YY MS Debt Monthly Yields
SwapMonthlyData <- read.table("data/SwapMonthlyData.csv",header = T,sep = ";",dec = ".", stringsAsFactors = FALSE)           ### Risk Free Rate used for pricing
MSList <- read.table("data/MSList2.csv",header = T,sep = ";",dec = ".", stringsAsFactors = FALSE)                             ### Load MS Treatment

Lieuro <- read_excel("data/LIEURO.xls")
MSList <- MSList[,-c(4,5)]

MyData <- merge(MSMonthlyRating, MSMonthlyDebt, by=c("MS", "Year", "Month"))
MyData <- merge(MyData, MS10YY , by=c("MS", "Year", "Month"))
MyData <- merge(MyData, SwapMonthlyData, by=c("Year", "Month"))
MyData <- merge(MyData, MSList, by="MS")
MyData <- merge(MyData, Lieuro, by=c("Year", "Month"))
MyData <- MyData[MyData$Exclude!="Y",]


TotalDebtByYearMonth <- aggregate(MyData[,5], by=list(MyData$Year, MyData$Month),FUN=sum, na.rm=TRUE)
colnames(TotalDebtByYearMonth) <- c("Year", "Month", "TotalDebt")
MyData <- merge(MyData, TotalDebtByYearMonth, by=c("Year", "Month"))

temp <- MyData[MyData$Aggregate=="Y",]
TotalOthersDebtByYearMonth <- aggregate(temp[,5], by=list(temp$Year, temp$Month),FUN=sum, na.rm=TRUE)
colnames(TotalOthersDebtByYearMonth) <- c("Year", "Month", "TotalOthersDebt")
MyData <- merge(MyData, TotalOthersDebtByYearMonth, by=c("Year", "Month"))
MyData <- MyData[,-9]
#ordering the data to have a consistent time-series dimension
MyData<-MyData[order(MyData$MS,MyData$Year,MyData$Month),]

rm(temp,TotalDebtByYearMonth, TotalOthersDebtByYearMonth, MSMonthlyDebt,MSMonthlyRating,
   SwapMonthlyData, MS10YY)

Grade <- c("AAA","AA", "A","BBB", "BB","B","CCC", "CC", "C","" )
#Position <- c("1","2", "3","4", "5","6","7", "7", "7", "7" )
#THIS COMMAND DOES NOT WORK IN RECENT VERSION OF R BECAUSE DOUBLE and not CHARACTER IS NEEDED
# THE FOLLOWING LINE WILL DO 
Position <- c(1,2,3,4,5,6,7,7,7,7)
Ratings <- data.frame(Grade, Position)


### Structural Input For Pricing
TTC <- read.table("data/TTC SOV.csv",header = F,sep = ";",dec = ".")
TTC <- as.matrix(TTC) #ncol=8, nrow=8)
D_TTC<-diag(eigen(TTC)$values)
S_TTC<-eigen(TTC)$vectors
S_TTC_Inv<-solve(S_TTC)

Test<-S_TTC%*%D_TTC%*%S_TTC_Inv
Test<-TTC-Test

UnitMatrix <- diag(1,8,8)
unitVector<- as.vector(rep.int(1, 8))
v <- as.vector(c(0,0,0,0,0,0,0,1))
rr <- .30

###### HERE CALIBRATION 
#Calibration - autoregression and theta2: 
AR<- ar.ols(Lieuro$LIEURO, 
            order.max = 1, 
            demean = F, 
            intercept = F)
THETA0<-AR$ar[1]
THETA2 <- -18.2
THETA1 <- as.vector(rep.int(0, 8))
for (i in 2:8) 
{THETA1[i] <- (1/D_TTC[i,i]-1)}

rm(AR,Test)

### TS Creation of the Fundamental Price By MS 
TestFirst <- 0

for (MS in MSList[MSList$Exclude=="N",1])
{
  MyDataByMS <- MyData[MyData$MS==MS,]
  TotalRecords <- dim(MyDataByMS)[1]
  MyDataByMS$UnitPrice <- 0
  MyDataByMS$UnitPriceWeighted <- 0
  MyDataByMS$OthersUnitPriceWeighted <- 0
  MyDataByMS$YieldWeighted <-0
  MyDataByMS$OthersYieldWeighted <-0
  #MyDataByMS$Aggregate <-"N"
  #MyDataByMS$Exclude <-"N"
  
  for (i in c(1:TotalRecords))
  {
    RF <- MyDataByMS[i,7]
    y <- MyDataByMS[i,9]  
    alpha <- 1/(1+RF)
    B <- D_TTC/(1+RF)
    # CREATE D_TM  
    
    #Opzione a) check: it  should be equivalent to TTC pricing 
    # D_TM <-D_TTC
    # D_TM1 <-D_TTC
    # D_TM2 <-D_TTC
    
    #segue alternativa b
    D_TM <- diag(1,8,8)
    D_TM1<- diag(1,8,8)
    D_TM2<- diag(1,8,8)
    for (j in 2:8) 
    {D_TM[j,j] <- as.double(1/(1+THETA1[j]*exp(THETA2*y))) 
    D_TM1[j,j] <- as.double(1/(1+THETA1[j]*exp(THETA2*THETA0^12*y)))
    D_TM2[j,j] <- as.double(1/(1+THETA1[j]*exp(THETA2*THETA0^24*y)))
    }
    
    B1 <- D_TM1/(1+RF)
    B2 <- D_TM2/(1+RF)
    
    Bfirst <- unitVector-diag(B)
    Bfirst <- B/Bfirst 
    #GROSS PERPETUITY 
    
    GrossPerpetuity <- alpha/(1-alpha)*unitVector
    #  GrossPerpetuity <- GrossPerpetuity-S_TTC%*%Bfirst%*%S_TTC_Inv%*%v ### eq. 8
    GrossPerpetuity <- GrossPerpetuity-S_TTC%*%(B1+B1%*%B2+B1%*%B2%*%Bfirst)%*%S_TTC_Inv%*%v
    
    #recovery rate 
    
    #D_TTCInv <- solve(D_TTC)
    #temp <- rr*S_TTC%*%((UnitMatrix-D_TTCInv)%*%Bfirst)%*%S_TTC_Inv%*%v ### eq. 9
    # here I redefine temp as rr_t 
    rr_t <- (rr/(1+RF)*S_TTC%*%(D_TM%*%D_TM1%*%(UnitMatrix-solve(D_TM1)))%*%S_TTC_Inv%*%v 
             +rr/(1+RF)^2*S_TTC%*%(D_TM%*%D_TM1%*%D_TM2%*%(UnitMatrix-solve(D_TM2)))%*%S_TTC_Inv%*%v 
             +rr/(1+RF)^2*S_TTC%*%(D_TM%*%D_TM1%*%D_TM2%*%(UnitMatrix-solve(D_TTC))%*%Bfirst)%*%S_TTC_Inv%*%v)
    
    
    temp <- (unitVector- rr_t)
    NetPerpetuity <- GrossPerpetuity/temp ### eq. 10
    cf <- 1/NetPerpetuity ### eq. 13 annual fundamental pricing
    
    MSCurrentRating <- MyDataByMS[i,4]
    PriceRecord <- Ratings[Ratings$Grade==MSCurrentRating,2]
    CurrentUnitPrice <- cf[PriceRecord]
    CurrentDebt <- MyDataByMS[i, 5]
    CurrentYield <- MyDataByMS[i, 6]
    TotalCurrentDebt <- MyDataByMS[i, 10]
    TotalOthersCurrentDebt <- MyDataByMS[i, 11]
    
    MyDataByMS[i, 12] <- CurrentUnitPrice
    MyDataByMS[i, 13] <- CurrentUnitPrice*CurrentDebt/TotalCurrentDebt
    MyDataByMS[i, 14] <- CurrentUnitPrice*CurrentDebt/TotalOthersCurrentDebt
    MyDataByMS[i, 15] <- CurrentYield*CurrentDebt/TotalCurrentDebt
    MyDataByMS[i, 16] <- CurrentYield*CurrentDebt/TotalOthersCurrentDebt
    
    
    #if (MSList[MSList$MS==MS,2]=="Y") MyDataByMS[i, 12] <- "Y"
    #if (MSList[MSList$MS==MS,3]=="Y") MyDataByMS[i, 13] <- "Y"
  }    
  
  if (TestFirst==0) 
  {
    TotalDataByMS <- MyDataByMS
    TestFirst <- 1
    
  } else {
    TotalDataByMS <- rbind(TotalDataByMS, MyDataByMS)
  } 
  
}

temp <- TotalDataByMS[,8]
TotalDataByMS <- TotalDataByMS[,-8]
TotalDataByMS$Aggregate <- temp

rm(MyDataByMS)

### Aggregate Others

temp <- TotalDataByMS[TotalDataByMS$Aggregate=="Y",]
OthersTotalDebt <- temp$TotalOthersDebt[1]
IR <- aggregate(temp[,7], by=list(temp$Year, temp$Month), FUN=mean, na.rm=TRUE)
temp$MS <-0
temp$Rating <- 0
temp$Aggregate <- 0
#temp$Exclude <- 0
temp <- aggregate(temp[,3:16], by=list(temp$Year, temp$Month), FUN=sum, na.rm=TRUE)
temp$MS <- "Others"
temp$Rating <- "AAA"
temp$Aggregate <- "N"
temp$Yield <- temp$OthersYieldWeighted
temp$UnitPrice <- temp$OthersUnitPriceWeighted
temp$TotalDebt <- OthersTotalDebt
temp$IR <- IR[,3]
colnames(temp) <- colnames(TotalDataByMS)

Others <- temp

### Aggregate Eurozone

temp <- TotalDataByMS
EurozoneTotalDebt <- temp$TotalDebt[1]
temp$MS <-0
temp$Rating <- 0
temp$Aggregate <- 0
#temp$Exclude <- 0
temp <- aggregate(temp[,3:16], by=list(temp$Year, temp$Month),FUN=sum, na.rm=TRUE)
temp$MS <- "Eurozone"
temp$Rating <- "AAA"
temp$Aggregate <- "N"
temp$Yield <- temp$YieldWeighted
temp$UnitPrice <- temp$UnitPriceWeighted
temp$TotalDebt <- EurozoneTotalDebt
temp$IR <- IR[,3]
colnames(temp) <- colnames(TotalDataByMS)

Eurozone <- temp

### Create total DB for pivot tables

TotalDataByMS <- rbind(TotalDataByMS, Others, Eurozone)
TotalDataByMS <- TotalDataByMS[,c(-8, -9, -10,-12,-13,-14,-15)]
TotalDataByMS<-TotalDataByMS[order(TotalDataByMS$MS,TotalDataByMS$Year,TotalDataByMS$Month),]

rm(Eurozone,IR, Others, temp)

write.table(TotalDataByMS,file="data/TotalDataByMS_2022.csv",row.names = F, sep = ";", dec='.')

#-------------------------------------------------------------------------------
### TS Creation of the Fundamental Price - WORST CASE 
#-------------------------------------------------------------------------------

MS = "AT (Austria)"
MyDataByMS <- MyData[MyData$MS==MS,]
MyDataByMS[,4] <- "C"
TotalRecords <- dim(MyDataByMS)[1]
MyDataByMS$UnitPrice <- 0
MyDataByMS$UnitPriceWeighted <- 0
MyDataByMS$OthersUnitPriceWeighted <- 0
MyDataByMS$YieldWeighted <-0
MyDataByMS$OthersYieldWeighted <-0
#MyDataByMS$Aggregate <-"N"
#MyDataByMS$Exclude <-"N"

for (i in c(1:TotalRecords))
{
  RF <- MyDataByMS[i,7]
  y <- MyDataByMS[i,9]  
  alpha <- 1/(1+RF)
  B <- D_TTC/(1+RF)
  # CREATE D_TM  
  
  #Opzione a) check: it  should be equivalent to TTC pricing 
  # D_TM <-D_TTC
  # D_TM1 <-D_TTC
  # D_TM2 <-D_TTC
  
  #segue alternativa b
  D_TM <- diag(1,8,8)
  D_TM1<- diag(1,8,8)
  D_TM2<- diag(1,8,8)
  for (j in 2:8) 
  {D_TM[j,j] <- as.double(1/(1+THETA1[j]*exp(THETA2*y))) 
  D_TM1[j,j] <- as.double(1/(1+THETA1[j]*exp(THETA2*THETA0^12*y)))
  D_TM2[j,j] <- as.double(1/(1+THETA1[j]*exp(THETA2*THETA0^24*y)))
  }
  
  B1 <- D_TM1/(1+RF)
  B2 <- D_TM2/(1+RF)
  
  Bfirst <- unitVector-diag(B)
  Bfirst <- B/Bfirst 
  #GROSS PERPETUITY 
  
  GrossPerpetuity <- alpha/(1-alpha)*unitVector
  #  GrossPerpetuity <- GrossPerpetuity-S_TTC%*%Bfirst%*%S_TTC_Inv%*%v ### eq. 8
  GrossPerpetuity <- GrossPerpetuity-S_TTC%*%(B1+B1%*%B2+B1%*%B2%*%Bfirst)%*%S_TTC_Inv%*%v
  
  #recovery rate 
  
  #D_TTCInv <- solve(D_TTC)
  #temp <- rr*S_TTC%*%((UnitMatrix-D_TTCInv)%*%Bfirst)%*%S_TTC_Inv%*%v ### eq. 9
  # here I redefine temp as rr_t 
  rr_t <- (rr/(1+RF)*S_TTC%*%(D_TM%*%D_TM1%*%(UnitMatrix-solve(D_TM1)))%*%S_TTC_Inv%*%v 
           +rr/(1+RF)^2*S_TTC%*%(D_TM%*%D_TM1%*%D_TM2%*%(UnitMatrix-solve(D_TM2)))%*%S_TTC_Inv%*%v 
           +rr/(1+RF)^2*S_TTC%*%(D_TM%*%D_TM1%*%D_TM2%*%(UnitMatrix-solve(D_TTC))%*%Bfirst)%*%S_TTC_Inv%*%v)
  
  
  temp <- (unitVector- rr_t)
  NetPerpetuity <- GrossPerpetuity/temp ### eq. 10
  cf <- 1/NetPerpetuity ### eq. 13 annual fundamental pricing
  
  MSCurrentRating <- MyDataByMS[i,4]
  PriceRecord <- Ratings[Ratings$Grade==MSCurrentRating,2]
  CurrentUnitPrice <- cf[PriceRecord]
  MyDataByMS[i, 12] <- CurrentUnitPrice
  
  
  
  #if (MSList[MSList$MS==MS,2]=="Y") MyDataByMS[i, 12] <- "Y"
  #if (MSList[MSList$MS==MS,3]=="Y") MyDataByMS[i, 13] <- "Y"
}    

UnitPrice_C <- MyDataByMS[,12]
dates <-seq(as.Date("2001-01-15"),length=nrow(MyDataByMS), by="months")
UnitPrice_C <- as.xts(UnitPrice_C, order.by=dates)
rm(MyDataByMS,temp)

#write.table(UnitPrice_C, file = "data/UnitPrice_C.csv", row.names = F, sep = ";", dec='.')

#-------------------------------------------------------------------------------
### TS Creation of Bands for Italy  
#-------------------------------------------------------------------------------

MS = "IT (Italy)"
MyDataByMS <- MyData[MyData$MS==MS,]
TotalRecords <- dim(MyDataByMS)[1]
MyDataByMS$UnitPrice <- 0
MyDataByMS$UnitPriceWeighted <- 0
MyDataByMS$OthersUnitPriceWeighted <- 0
MyDataByMS$YieldWeighted <-0
MyDataByMS$OthersYieldWeighted <-0
#MyDataByMS$Aggregate <-"N"
#MyDataByMS$Exclude <-"N"

for (i in c(1:TotalRecords))
{
  RF <- MyDataByMS[i,7]
  y <- MyDataByMS[i,9]  
  alpha <- 1/(1+RF)
  B <- D_TTC/(1+RF)
  # CREATE D_TM  
  
  #Opzione a) check: it  should be equivalent to TTC pricing 
  # D_TM <-D_TTC
  # D_TM1 <-D_TTC
  # D_TM2 <-D_TTC
  
  #segue alternativa b
  D_TM <- diag(1,8,8)
  D_TM1<- diag(1,8,8)
  D_TM2<- diag(1,8,8)
  for (j in 2:8) 
  {D_TM[j,j] <- as.double(1/(1+THETA1[j]*exp(THETA2*y))) 
  D_TM1[j,j] <- as.double(1/(1+THETA1[j]*exp(THETA2*THETA0^12*y)))
  D_TM2[j,j] <- as.double(1/(1+THETA1[j]*exp(THETA2*THETA0^24*y)))
  }
  
  B1 <- D_TM1/(1+RF)
  B2 <- D_TM2/(1+RF)
  
  Bfirst <- unitVector-diag(B)
  Bfirst <- B/Bfirst 
  #GROSS PERPETUITY 
  
  GrossPerpetuity <- alpha/(1-alpha)*unitVector
  #  GrossPerpetuity <- GrossPerpetuity-S_TTC%*%Bfirst%*%S_TTC_Inv%*%v ### eq. 8
  GrossPerpetuity <- GrossPerpetuity-S_TTC%*%(B1+B1%*%B2+B1%*%B2%*%Bfirst)%*%S_TTC_Inv%*%v
  
  #recovery rate 
  
  #D_TTCInv <- solve(D_TTC)
  #temp <- rr*S_TTC%*%((UnitMatrix-D_TTCInv)%*%Bfirst)%*%S_TTC_Inv%*%v ### eq. 9
  # here I redefine temp as rr_t 
  rr_t <- (rr/(1+RF)*S_TTC%*%(D_TM%*%D_TM1%*%(UnitMatrix-solve(D_TM1)))%*%S_TTC_Inv%*%v 
           +rr/(1+RF)^2*S_TTC%*%(D_TM%*%D_TM1%*%D_TM2%*%(UnitMatrix-solve(D_TM2)))%*%S_TTC_Inv%*%v 
           +rr/(1+RF)^2*S_TTC%*%(D_TM%*%D_TM1%*%D_TM2%*%(UnitMatrix-solve(D_TTC))%*%Bfirst)%*%S_TTC_Inv%*%v)
  
  
  temp <- (unitVector- rr_t)
  NetPerpetuity <- GrossPerpetuity/temp ### eq. 10
  cf <- 1/NetPerpetuity ### eq. 13 annual fundamental pricing
  
  MSCurrentRating <- MyDataByMS[i,4]
  PriceRecord <- Ratings[Ratings$Grade==MSCurrentRating,2]
  PriceRecord <- (PriceRecord+1)
  CurrentUnitPrice <- cf[PriceRecord]
  MyDataByMS[i, 12] <- CurrentUnitPrice
  
  
  
  #if (MSList[MSList$MS==MS,2]=="Y") MyDataByMS[i, 12] <- "Y"
  #if (MSList[MSList$MS==MS,3]=="Y") MyDataByMS[i, 13] <- "Y"
}    

Upper_class_IT <- MyDataByMS[,12]
dates <-seq(as.Date("2001-01-15"),length=nrow(MyDataByMS), by="months")
Upper_class_IT<- as.xts(Upper_class_IT, order.by=dates)
rm(MyDataByMS,temp)

MS = "IT (Italy)"
MyDataByMS <- MyData[MyData$MS==MS,]
TotalRecords <- dim(MyDataByMS)[1]
MyDataByMS$UnitPrice <- 0
MyDataByMS$UnitPriceWeighted <- 0
MyDataByMS$OthersUnitPriceWeighted <- 0
MyDataByMS$YieldWeighted <-0
MyDataByMS$OthersYieldWeighted <-0
#MyDataByMS$Aggregate <-"N"
#MyDataByMS$Exclude <-"N"

for (i in c(1:TotalRecords))
{
  RF <- MyDataByMS[i,7]
  y <- MyDataByMS[i,9]  
  alpha <- 1/(1+RF)
  B <- D_TTC/(1+RF)
  # CREATE D_TM  
  
  #Opzione a) check: it  should be equivalent to TTC pricing 
  # D_TM <-D_TTC
  # D_TM1 <-D_TTC
  # D_TM2 <-D_TTC
  
  #segue alternativa b
  D_TM <- diag(1,8,8)
  D_TM1<- diag(1,8,8)
  D_TM2<- diag(1,8,8)
  for (j in 2:8) 
  {D_TM[j,j] <- as.double(1/(1+THETA1[j]*exp(THETA2*y))) 
  D_TM1[j,j] <- as.double(1/(1+THETA1[j]*exp(THETA2*THETA0^12*y)))
  D_TM2[j,j] <- as.double(1/(1+THETA1[j]*exp(THETA2*THETA0^24*y)))
  }
  
  B1 <- D_TM1/(1+RF)
  B2 <- D_TM2/(1+RF)
  
  Bfirst <- unitVector-diag(B)
  Bfirst <- B/Bfirst 
  #GROSS PERPETUITY 
  
  GrossPerpetuity <- alpha/(1-alpha)*unitVector
  #  GrossPerpetuity <- GrossPerpetuity-S_TTC%*%Bfirst%*%S_TTC_Inv%*%v ### eq. 8
  GrossPerpetuity <- GrossPerpetuity-S_TTC%*%(B1+B1%*%B2+B1%*%B2%*%Bfirst)%*%S_TTC_Inv%*%v
  
  #recovery rate 
  
  #D_TTCInv <- solve(D_TTC)
  #temp <- rr*S_TTC%*%((UnitMatrix-D_TTCInv)%*%Bfirst)%*%S_TTC_Inv%*%v ### eq. 9
  # here I redefine temp as rr_t 
  rr_t <- (rr/(1+RF)*S_TTC%*%(D_TM%*%D_TM1%*%(UnitMatrix-solve(D_TM1)))%*%S_TTC_Inv%*%v 
           +rr/(1+RF)^2*S_TTC%*%(D_TM%*%D_TM1%*%D_TM2%*%(UnitMatrix-solve(D_TM2)))%*%S_TTC_Inv%*%v 
           +rr/(1+RF)^2*S_TTC%*%(D_TM%*%D_TM1%*%D_TM2%*%(UnitMatrix-solve(D_TTC))%*%Bfirst)%*%S_TTC_Inv%*%v)
  
  
  temp <- (unitVector- rr_t)
  NetPerpetuity <- GrossPerpetuity/temp ### eq. 10
  cf <- 1/NetPerpetuity ### eq. 13 annual fundamental pricing
  
  MSCurrentRating <- MyDataByMS[i,4]
  PriceRecord <- Ratings[Ratings$Grade==MSCurrentRating,2]
  PriceRecord <- (PriceRecord-1)
  CurrentUnitPrice <- cf[PriceRecord]
  MyDataByMS[i, 12] <- CurrentUnitPrice
  
  
  
  #if (MSList[MSList$MS==MS,2]=="Y") MyDataByMS[i, 12] <- "Y"
  #if (MSList[MSList$MS==MS,3]=="Y") MyDataByMS[i, 13] <- "Y"
}    

Lower_class_IT <- MyDataByMS[,12]
dates <-seq(as.Date("2001-01-15"),length=nrow(MyDataByMS), by="months")
Lower_class_IT<- as.xts(Lower_class_IT, order.by=dates)
rm(MyDataByMS,temp)

#-------------------------------------------------------------------------------
###GRAPHICS: 
#-------------------------------------------------------------------------------
#FIGURE 1: 10Y YIELDS
#-------------------------------------------------------------------------------

x <- TotalDataByMS[TotalDataByMS$MS=="FR (France)",6]
x <- as.xts(x, order.by=dates)
x <- cbind(x,  TotalDataByMS[TotalDataByMS$MS=="IT (Italy)",6], 
               TotalDataByMS[TotalDataByMS$MS=="DE (Germany)",6], 
               TotalDataByMS[TotalDataByMS$MS=="ES (Spain)",6],
               TotalDataByMS[TotalDataByMS$MS=="GR (Greece)",6]
               #TotalDataByMS[TotalDataByMS$MS=="Others",6],
               #TotalDataByMS[TotalDataByMS$MS=="Eurozone",6],
               #TotalDataByMS[TotalDataByMS$MS=="Eurozone",7]
           )
colnames(x)<- c("FR", "IT", "DE", "ES","GR" 
                #"Others", "Eurozone", "IR"
                )
dates <- index(x)

x <- as.data.frame(x)
x <- cbind(x, dates) 
x <- melt(x,id.vars=c("dates"),value.name="value", variable.name="Country")


ggplot(data = x, aes(x = dates, y = value, color = Country, group = Country)) +
  geom_line(size = 0.7) +
  scale_color_brewer(palette = "Set1") +
  labs(x = "", y = "", title = "Market Yield 10-Year Government Bonds") +
  scale_x_date(date_labels = "%m-%Y", 
               breaks = "24 month", 
               limits = as.Date(c("2001-01-15", "2021-12-15")), 
               expand = c(0, 0)) +
  theme_minimal() +
  theme(legend.position = c(0.95, 0.95),
        legend.justification = c("right", "top"),
        legend.box.just = "right",
        legend.margin = margin(6, 6, 6, 6))

dev.copy2pdf(width = 8, out.type = "pdf",file="figures/Figure_1_a.pdf")
dev.off()

#ggsave("figures/Figure_1.eps", device = "eps")


#-------------------------------------------------------------------------------
# FIGURE 2: INTEREST RATES ON EDA LOANS 
#-------------------------------------------------------------------------------

x <-TotalDataByMS[TotalDataByMS$MS=="IT (Italy)",8]
x<- as.xts(x, order.by=dates)
x <- cbind(x, TotalDataByMS[TotalDataByMS$MS=="DE (Germany)",8], 
              TotalDataByMS[TotalDataByMS$MS=="IT (Italy)",6], 
              TotalDataByMS[TotalDataByMS$MS=="DE (Germany)",6],
              UnitPrice_C[,1])
colnames(x)<- c("IT - Interest rate on EDA Loans",
                "DE - Interest rate on EDA Loans",
                "IT - 10Y mkt yield",
                "DE - 10Y mkt yield",
                "Worst case - Interest rate on EDA Loans")
dates <- index(x)

x <- as.data.frame(x)
x <- cbind(x, dates) 
x <- melt(x,id.vars=c("dates"),value.name="value", variable.name="Yields")

ggplot(data = x, aes(x = dates, y = value, linetype = Yields, color = Yields, group = Yields)) +
  geom_line(size = 0.7) +
  scale_color_manual(values = c("red", "blue", "red", "blue", "black")) +
  scale_linetype_manual(values = c("solid", "solid","dashed", "dashed", "solid")) +
  labs(x = "", y = "", title = "10-Year Market Yields and Simulated Interest rates on EDA Loans") +
  scale_x_date(date_labels = "%Y-%m", 
               breaks = "24 month", 
               limits = as.Date(c("2001-01-15", "2021-12-15")), 
               expand = c(0, 0)) +
  theme_minimal()+
  theme(legend.position = c(0.95, 0.95),
        legend.justification = c("right", "top"),
        legend.box.just = "right",
        legend.margin = margin(6, 6, 6, 6))

dev.copy2pdf(width = 8, out.type = "pdf",file="figures/Figure_1_b.pdf")
dev.off()


#ggsave("figures/Figure_2.eps", device = "eps") 


#-------------------------------------------------------------------------------
#FIGURE 3: 10Y YIELDS, FAIR VALUE and Interest rates on EDA Loans
#-------------------------------------------------------------------------------
x <-TotalDataByMS[TotalDataByMS$MS=="IT (Italy)",8]
x<- as.xts(x, order.by=dates)
x <- cbind(x, TotalDataByMS[TotalDataByMS$MS=="DE (Germany)",8], 
           TotalDataByMS[TotalDataByMS$MS=="IT (Italy)",6], 
           TotalDataByMS[TotalDataByMS$MS=="DE (Germany)",6],
           UnitPrice_C[,1])
colnames(x)<- c("Italy - Interest rate on EDA Loans",
                "Germany - Interest rate on EDA Loans",
                "Italy - 10Y mkt yield",
                "Germany - 10Y mkt yield",
                "Worst case - Interest rate on EDA Loans")
dates <- index(x)

x <- as.data.frame(x)
x <- cbind(x, dates) 
x <- melt(x,id.vars=c("dates"),value.name="value", variable.name="Yields")

x <- x[x$dates >= "2007-01-15" & x$dates <= "2022-06-15", ]

Bands  <- read_excel("data/Bands_202207.xlsx")
dates3 <- seq(as.Date("2007-01-15"),length=(186), by="months")
Bands  <- as.xts(Bands , order.by=dates3)

dates3 <- index(Bands)
Bands  <- Bands [,-1]
Bands  <- as.data.frame(Bands)

Lower_class_IT <- as.data.frame(Lower_class_IT)
Lower_class_IT <- cbind(Lower_class_IT, dates)
Lower_class_IT <- Lower_class_IT[Lower_class_IT$dates >= "2007-01-15" & Lower_class_IT$dates <= "2022-06-15", ]
Lower_class_IT <- Lower_class_IT[,-2]

Upper_class_IT <- as.data.frame(Upper_class_IT)
Upper_class_IT <- cbind(Upper_class_IT, dates)
Upper_class_IT <- Upper_class_IT[Upper_class_IT$dates >= "2007-01-15" & Upper_class_IT$dates <= "2022-06-15", ]
Upper_class_IT <- Upper_class_IT[,-2]


itamrk <- x[x$Yields=="Italy - 10Y mkt yield", 3]
itaeda <- x[x$Yields=="Italy - Interest rate on EDA Loans", 3]
germrk <- x[x$Yields=="Germany - 10Y mkt yield", 3]


Bands  <- cbind(Bands, itamrk, itaeda, germrk, Lower_class_IT, Upper_class_IT, dates3)


Bands$Fair_spread = as.numeric(Bands$Fair_spread)
Bands$p5_Bootstrap = as.numeric(Bands$p5_Bootstrap)
Bands$p95_Bootstrap = as.numeric(Bands$p95_Bootstrap)

Bands  <- Bands %>%
  mutate(Fair_spread = (Fair_spread/10000 +germrk)) %>%
  mutate(p5_Bootstrap= (p5_Bootstrap/10000 +germrk)) %>%
  mutate(p95_Bootstrap= (p95_Bootstrap/10000+germrk))

Bands = Bands %>%
  select("dates3",
         "Fair_spread",
         "p5_Bootstrap",
         "p95_Bootstrap",
         "itamrk",
         "itaeda",
         "Lower_class_IT",
         "Upper_class_IT")

colnames(Bands)<- c("dates3",
                    "Fair Value 10YY",
                    "p5_Fair Value",
                    "p95_Fair Value",
                    "Observed 10YY",
                    "Interest Rate on EDA Loans",
                    "Lower_Interest Rate on EDA Loans",
                    "Upper_Interest Rate on EDA Loans")

Bands <- melt(Bands,id.vars=c("dates3"),value.name="value", variable.name="Yields")


ggplot(data = Bands, aes(x = dates3, y = value, linetype = Yields, color = Yields, group = Yields)) +
  geom_line(size = 0.7) +
  scale_color_manual(values = c("red", "red", "red", "blue", "green", "green", "green")) +
  scale_linetype_manual(values = c("solid", "dashed","dashed", "solid", "solid", "dashed","dashed")) +
  labs(x = "", y = "", title = "Yields - Italy") +
  scale_x_date(date_labels = "%Y-%m", 
               breaks = "24 month", 
               limits = as.Date(c("2007-01-15", "2022-06-15")), 
               expand = c(0, 0)) +
  theme_minimal()+
  theme(legend.position = c(0.95, 0.95),
        legend.justification = c("right", "top"),
        legend.box.just = "right",
        legend.margin = margin(6, 6, 6, 6))

dev.copy2pdf(width = 8, out.type = "pdf",file="figures/Figure_2.pdf")
dev.off()

#ggsave("figures/Figure_3.eps", device = "eps")



