# clear memory 
rm(list=ls())
#Check the working directory before importing else provide full path
#setwd(path)
setwd(dirname(rstudioapi::getActiveDocumentContext()$path))
# packages used


listofpackages <- c("dygraphs", "dplyr","ellipse","reshape2","DAAG","ggplot2","stringr","xts","readxl","stats",
                    "plotly", "hrbrthemes", "zoo", "dynlm")

for (j in listofpackages){
  if(sum(installed.packages()[, 1] == j) == 0) {
    install.packages(j)
  }
  library(j, character.only = T)
}


##### N.B. DEFICIT IN THE DATA  is (t-g)

##### DATABASE IN PANEL FORMAT: 

Data_S0 <- read.table("data/Data_Baseline_bund.csv",header = T,sep = ";",dec = ".", stringsAsFactors = FALSE) 
Data_S1 <- read.table("data/Data_EDA_scenario_bund.csv",header = T,sep = ";",dec = ".", stringsAsFactors = FALSE) 
Stochastic_EDA <- read.table("data/EDA_2_scenarios_bund.csv",header = T,sep = ";",dec = ".", stringsAsFactors = FALSE) 
MSList <- read.table("data/MSList2.csv",header = T,sep = ";",dec = ".", stringsAsFactors = FALSE)

Data_S0_sigma <- read.table("data/Data_Baseline_bund_sigma.csv",header = T,sep = ";",dec = ".", stringsAsFactors = FALSE) 
Data_S1_sigma <- read.table("data/Data_EDA_scenario_bund_sigma.csv",header = T,sep = ";",dec = ".", stringsAsFactors = FALSE) 

dates <-seq(as.Date("2022-12-15"),length=21, by="years")
Stochastic_EDA <- as.xts(Stochastic_EDA, order.by=dates)
Data_S0 <- as.xts(Data_S0, order.by=dates)
Data_S1 <- as.xts(Data_S1, order.by=dates)

Data_S0_sigma <- as.xts(Data_S0_sigma, order.by=dates)
Data_S1_sigma <- as.xts(Data_S1_sigma, order.by=dates)
#-------------------------------------------------------------------------------
### GRAPHS:  
#-------------------------------------------------------------------------------

#-----------------------------
### Growth
#-----------------------------

plot(Data_S0_sigma$IT_g_bt_mean, main = "Growth - Italy", 
     type = "l", col = "blue",ylim = c(-0.04, 0.09), cex=0.7)
lines(Data_S0_sigma$IT_g_bt_q95,col = "blue",lwd=2, lty=2)
lines(Data_S1_sigma$IT_g_bt_mean, col="green",lwd=2)
lines(Data_S1_sigma$IT_g_bt_q95,col = "green",lwd=2, lty=2)

addLegend("topright",
          legend.names = c( "Baseline scenario", "Small EDA"),
          col=c("blue", "green"), lty=c(1), lwd=2, text.font=3)


#-----------------------------
### Swap, IC, Avg Cost - ITALY: 
#-----------------------------

### NO EDA 
plot(Stochastic_EDA$IR10_bt_mean, main = "Swap Rate, Avg Cost, 10Y - 
     Italy (Baseline Scenario)", 
     type = "l", col = "blue",ylim = c(0.01, 0.1), cex=0.7)
lines(Stochastic_EDA$IR10_bt_q95, col = "blue", lty=2,lwd=2)
lines(Data_S0_sigma$IT_r_10Y_bt_mean, col="orange",lwd=2)
lines(Data_S0_sigma$IT_r_10Y_bt_q95, col="orange",lty=2,lwd=2)
lines(Data_S0_sigma$IT_r_bt_mean, col = "red",lwd=2)
lines(Data_S0_sigma$IT_r_bt_q95,col = "red",lwd=2, lty=2)



addLegend("topright",
          legend.names = c( "Swap rate", "10Y rate", "Avg Cost"),
          col=c("blue", "orange", "red"), lty=c(1), lwd=2, text.font=3)


### Small EDA
plot(Stochastic_EDA$IR10_bt_EDA_mean, main = "Swap Rate, Avg Cost, IC - 
     Italy (Small EDA)", 
     type = "l", col = "blue",ylim = c(-0.02, 0.07), cex=0.7)
lines(Stochastic_EDA$IR10_bt_EDA_q95, col = "blue", lty=2,lwd=2)
lines(Data_S1_sigma$IT_r_10Y_bt_mean, col="orange",lwd=2)
lines(Data_S1_sigma$IT_r_10Y_bt_q95, col="orange",lty=2,lwd=2)
lines(Data_S1_sigma$IT_r_bt_mean, col = "red",lwd=2)
lines(Data_S1_sigma$IT_r_bt_q95,col = "red",lwd=2, lty=2)
lines(Data_S1_sigma$IT_IC_mean, col="green", lwd=2)
lines(Data_S1_sigma$IT_IC_q95, col="green", lwd=2, lty=2)


addLegend("topright",
          legend.names = c( "Swap rate", "10Y rate", "Avg Cost", "Rate on EDA Loans"),
          col=c("blue", "orange","red", "green"), lty=c(1), lwd=2, text.font=3)


# dev.copy2pdf(width = 8, out.type = "pdf",file="figures/Figure_3_a.pdf")
# dev.off()


#-----------------------------
### Swap, IC, Avg Cost - Germany: 
#-----------------------------
plot(Stochastic_EDA$IR10_bt_EDA_mean, main = "Swap Rate, Avg Cost, IC - Germany", type = "l", 
     col = "blue",ylim = c(-0.04, 0.06), cex=0.7)
lines(Stochastic_EDA$IR10_bt_EDA_q95, col = "blue", lty=2,lwd=2)
lines(Data_S1_sigma$DE_r_10Y_bt_mean, col="orange",lwd=2)
lines(Data_S1_sigma$DE_r_10Y_bt_q95, col="orange",lty=2,lwd=2)
lines(Data_S1_sigma$DE_r_bt_mean, col = "red",lwd=2)
lines(Data_S1_sigma$DE_r_bt_q95,col = "red",lwd=2, lty=2)
lines(Data_S1_sigma$DE_IC_mean, col="green", lwd=2)
lines(Data_S1_sigma$DE_IC_q95, col="green", lwd=2, lty=2)


addLegend("topright",
          legend.names = c( "Swap rate", "10Y rate", "Avg Cost", "Rate on EDA Loans"),
          col=c("blue", "orange","red", "green"), lty=c(1), lwd=2, text.font=3)


# dev.copy2pdf(width = 8, out.type = "pdf",file="figures/Figure_3_b.pdf")
# dev.off()


#-------------------------------------------------------------------------------
### Total DEBT/GDP and Primary Surplus:  
#-------------------------------------------------------------------------------

#-------- DEBT-----------

library(ggplot2)

# Create a data frame with example data

df<- cbind(Data_S0$IT_b_bt_mean, Data_S1$IT_d_bt_mean, Data_S0$DE_b_bt_mean,Data_S1$DE_d_bt_mean,
           Data_S0$FR_b_bt_mean, Data_S1$FR_d_bt_mean, Data_S0$ES_b_bt_mean, Data_S1$ES_d_bt_mean,
           Data_S0$GR_b_bt_mean, Data_S1$GR_d_bt_mean, Data_S0$PT_b_bt_mean, Data_S1$PT_d_bt_mean,
           
           Data_S0_sigma$IT_b_bt_mean, Data_S0_sigma$DE_b_bt_mean,
           Data_S0_sigma$FR_b_bt_mean, Data_S0_sigma$ES_b_bt_mean,
           Data_S0_sigma$GR_b_bt_mean, Data_S0_sigma$PT_b_bt_mean,
           
           Data_S1_sigma$IT_d_bt_mean, Data_S1_sigma$DE_d_bt_mean, Data_S1_sigma$ES_d_bt_mean,
           Data_S1_sigma$FR_d_bt_mean, Data_S1_sigma$GR_d_bt_mean, Data_S1_sigma$PT_d_bt_mean)
df <- as.data.frame(df) 

date = seq(as.Date("2022-12-15"), length.out = length(Data_S0$IT_b_bt_mean), by = "year")
  
colors <- c("Baseline Scenario" = "blue", "Baseline Scenario (sigma)" = "purple", "Small EDA" = "green",
            "Small EDA w/Sigma"= "red")


#Define your theme: 
my_theme <- theme_minimal() + theme(
  plot.title = element_text(size = 13, ),
  legend.position = c(0.95, 0.95),
  legend.justification = c("right", "top"),
  legend.box.just = "right",
  legend.margin = margin(6, 6, 6, 6),
  legend.title = element_blank(),
  legend.text = element_text(size = 10),
  axis.text = element_text(size = 10),
  axis.title.y = element_text(size = 11),
  panel.background = element_blank() )


#------------------------
####ITALY:

# Plot the lines
IT_Debt <- ggplot(data = df, aes(x = date)) +
  geom_line(aes(y = IT_b_bt_mean.1, color = "Baseline Scenario"), size = 0.6) +
  geom_line(aes(y = IT_d_bt_mean.1, color = "Small EDA"), size = 0.6) +
  scale_x_date( date_breaks = "3 years", date_labels = "%b\n%Y",
    limits = as.Date(c("2022-12-01", "2042-12-15")) ) +
  scale_y_continuous(limits = c(0.6, 1.8),breaks = seq(0.6, 1.8, 0.2) ) +
  ylab(NULL) +
  xlab(NULL) +
  labs(title = bquote(bold("Italy"))) +
  scale_color_manual(
    name = "Legend", 
    values = c("Baseline Scenario" = "blue", "Small EDA" = "green"),
    labels = c("Baseline Scenario", "Small EDA") ) +
  my_theme

IT_Debt

#------------------------
####GERMANY:

# Plot the lines
DE_Debt <- ggplot(df, aes(x = date)) +
  geom_line(aes(y = DE_b_bt_mean.1, color = "Baseline Scenario"), size = 0.6) +
  geom_line(aes(y = DE_d_bt_mean.1, color = "Small EDA"), size = 0.6) +
  scale_x_date( date_breaks = "3 years", date_labels = "%b\n%Y"
  ) +                                    # Set the x-axis labels as dates with two lines
  scale_y_continuous(
    limits = c(0.6, 1.8),              # Imposta i limiti dell'asse y
    breaks = seq(0.6, 1.8, 0.2)       # Imposta i punti di rottura dell'asse y
  ) +
  ylab(NULL) +                           # Set the y-axis label
  xlab(NULL) +                           # Remove x-axis label
  labs(title = bquote(bold("Germany"))) +       # Set the plot title
  scale_color_manual(
    name = "Legend",                     # Add a legend
    values = c("Baseline Scenario" = "blue", "Small EDA" = "green"),
    labels = c("Baseline Scenario", "Small EDA") ) +
  my_theme

DE_Debt

#------------------------
####SPAIN:

# Plot the lines
ES_Debt <- ggplot(df, aes(x = date)) +
  geom_line(aes(y = ES_b_bt_mean.1, color = "Baseline Scenario"), size = 0.6) +
  geom_line(aes(y = ES_d_bt_mean.1, color = "Small EDA"), size = 0.6) +
  scale_x_date(date_breaks = "3 years", date_labels = "%b\n%Y"
  ) +                     
  scale_y_continuous(limits = c(0.6, 1.8), breaks = seq(0.6, 1.8, 0.20)     
  ) +
  ylab(NULL) +   
  xlab(NULL) +                  
  labs(title = bquote(bold("Spain"))) +      
  scale_color_manual(
    name = "Legend",            # Add a legend
    values = c("Baseline Scenario" = "blue", "Small EDA" = "green"),
    labels = c("Baseline Scenario", "Small EDA") ) +
  my_theme

ES_Debt

#------------------------
####FRANCE:

# Plot the lines
FR_Debt <- ggplot(df, aes(x = date)) +
  geom_line(aes(y = FR_b_bt_mean.1, color = "Baseline Scenario"), size = 0.6) +
  geom_line(aes(y = FR_d_bt_mean.1, color = "Small EDA"), size = 0.6) +
  scale_x_date(
    date_breaks = "3 years",
    date_labels = "%b\n%Y") +           
  scale_y_continuous(
    limits = c(0.6, 1.8), breaks = seq(0.6, 1.8, 0.20)) +
  ylab(NULL) +  
  xlab(NULL) +               
  labs(title = bquote(bold("France"))) +       
  scale_color_manual( name = "Legend",           
  values = c("Baseline Scenario" = "blue", "Small EDA" = "green"),
  labels = c("Baseline Scenario", "Small EDA") ) +
  my_theme

FR_Debt

#------------------------
####GREECE:

# Plot the lines
GR_Debt <- ggplot(df, aes(x = date)) +
  geom_line(aes(y = GR_b_bt_mean.1, color = "Baseline Scenario"), size = 0.6) +
  geom_line(aes(y = GR_d_bt_mean.1, color = "Small EDA"), size = 0.6) +
  
  scale_x_date(
    date_breaks = "3 years",
    date_labels = "%b\n%Y") +          
  scale_y_continuous(
    limits = c(0.6, 1.8), breaks = seq(0.6, 1.8, 0.20) ) +
  ylab(NULL) +   
  xlab(NULL) +                 
  labs(title = bquote(bold("Greece"))) +      
  scale_color_manual(
    name = "Legend",           
    values = c("Baseline Scenario" = "blue", "Small EDA" = "green"),
    labels = c("Baseline Scenario", "Small EDA") ) +
  my_theme

GR_Debt

#------------------------
####PORTUGAL:

# Plot the lines
PT_Debt <- ggplot(df, aes(x = date)) +
  geom_line(aes(y = PT_b_bt_mean.1, color = "Baseline Scenario"), size = 0.6) +
  geom_line(aes(y = PT_d_bt_mean.1, color = "Small EDA"), size = 0.6) +
  scale_x_date(
    date_breaks = "3 years",
    date_labels = "%b\n%Y" ) +                
  scale_y_continuous(
    limits = c(0.6, 1.8),breaks = seq(0.6, 1.8, 0.20) ) +
  ylab(NULL) +  
  xlab(NULL) +                  
  labs(title = bquote(bold("Portugal"))) +      
  scale_color_manual(name = "Legend",            
  values = c("Baseline Scenario" = "blue", "Small EDA" = "green"),
  labels = c("Baseline Scenario", "Small EDA") ) +
  my_theme

PT_Debt

### Combine plots:
library(gridExtra)


grid.arrange(DE_Debt, ES_Debt, FR_Debt,
             GR_Debt, IT_Debt, PT_Debt, ncol = 2)

dev.copy2pdf(width = 6.5, height=9, out.type = "pdf",file="figures/Figure_Debt.eps")
dev.off()

#-------- PRIMARY SURPLUS -----------

df1 <- data.frame(
  date = seq(as.Date("2022-12-15"), length.out = length(Data_S0_sigma$IT_PrSur_bt_mean), by = "year"),
  
  Data_S0_sigma$IT_PrSur_bt_mean, Data_S0_sigma$IT_PrSur_bt_q95, Data_S1$IT_PrSur_bt_mean, Data_S1$IT_PrSur_bt_q95,
  Data_S1_sigma$IT_PrSur_bt_mean, Data_S1_sigma$IT_PrSur_bt_q95,
  
  Data_S0_sigma$ES_PrSur_bt_mean, Data_S0_sigma$ES_PrSur_bt_q95, Data_S1$ES_PrSur_bt_mean, Data_S1$ES_PrSur_bt_q95,
  Data_S1_sigma$ES_PrSur_bt_mean, Data_S1_sigma$ES_PrSur_bt_q95,
  
  Data_S0_sigma$PT_PrSur_bt_mean, Data_S0_sigma$PT_PrSur_bt_q95, Data_S1$PT_PrSur_bt_mean, Data_S1$PT_PrSur_bt_q95,
  Data_S1_sigma$PT_PrSur_bt_mean, Data_S1_sigma$PT_PrSur_bt_q95,
  
  Data_S0_sigma$GR_PrSur_bt_mean, Data_S0_sigma$GR_PrSur_bt_q95, Data_S1$GR_PrSur_bt_mean, Data_S1$GR_PrSur_bt_q95,
  Data_S1_sigma$GR_PrSur_bt_mean, Data_S1_sigma$GR_PrSur_bt_q95,
  
  Data_S0_sigma$DE_PrSur_bt_mean, Data_S0_sigma$DE_PrSur_bt_q95, Data_S1$DE_PrSur_bt_mean, Data_S1$DE_PrSur_bt_q95,
  Data_S1_sigma$DE_PrSur_bt_mean, Data_S1_sigma$DE_PrSur_bt_q95,
  
  Data_S0_sigma$FR_PrSur_bt_mean, Data_S0_sigma$FR_PrSur_bt_q95, Data_S1$FR_PrSur_bt_mean, Data_S1$FR_PrSur_bt_q95,
  Data_S1_sigma$FR_PrSur_bt_mean, Data_S1_sigma$FR_PrSur_bt_q95
  
  
)

colors <- c("Baseline Scenario" = "blue", "Small EDA" = "green",
            "Small EDA w/Sigma"= "red")

#------------------------
####ITALY:

IT_Surplus <- ggplot(df1, aes(x = date)) +
  geom_line(aes(y = IT_PrSur_bt_mean, color = "Baseline Scenario"), size = 0.6) +
  geom_line(aes(y = IT_PrSur_bt_q95, color = "Baseline Scenario"), size = 0.6, linetype = "dashed") +
  geom_line(aes(y = IT_PrSur_bt_mean.2, color = "Small EDA"), size = 0.6) +
  geom_line(aes(y = IT_PrSur_bt_q95.2, color = "Small EDA"), size = 0.6, linetype = "dashed") +
  scale_x_date(
    date_breaks = "3 years",
    date_labels = "%b\n%Y",
    limits = as.Date(c("2022-12-01", "2042-12-15"))
  ) +
  scale_y_continuous(
    limits = c(-0.05, 0.25),
    breaks = seq(-0.05, 0.25, 0.03)
  ) +
  ylab(NULL) +
  xlab(NULL) +
  labs(title = bquote(bold("Italy"))) +
  scale_color_manual(
    name = "Legend",
    values = c("Baseline Scenario" = "blue", "Small EDA" = "green"),
    labels = c("Baseline Scenario", "Small EDA")   
  ) +
  my_theme

IT_Surplus

#------------------------
####GERMANY:

DE_Surplus <- ggplot(df1, aes(x = date)) +
  geom_line(aes(y = DE_PrSur_bt_mean, color = "Baseline Scenario"), size = 0.6) +
  geom_line(aes(y = DE_PrSur_bt_q95, color = "Baseline Scenario"), size = 0.6, linetype = "dashed") +
  geom_line(aes(y = DE_PrSur_bt_mean.2, color = "Small EDA"), size = 0.6) +
  geom_line(aes(y = DE_PrSur_bt_q95.2, color = "Small EDA"), size = 0.6, linetype = "dashed") +
  scale_x_date(
    date_breaks = "3 years",
    date_labels = "%b\n%Y",
    limits = as.Date(c("2022-12-01", "2042-12-15"))
  ) +
  scale_y_continuous(
    limits = c(-0.05, 0.25),
    breaks = seq(-0.05, 0.25, 0.03)
  ) +
  ylab(NULL) +
  xlab(NULL) +
  labs(title = bquote(bold("Germany"))) +
  scale_color_manual(
    name = "Legend",
    values = c("Baseline Scenario" = "blue", "Small EDA" = "green"),
    labels = c("Baseline Scenario", "Small EDA")   
  ) +
  my_theme

DE_Surplus

#------------------------
####SPAIN:

ES_Surplus <- ggplot(df1, aes(x = date)) +
  geom_line(aes(y = ES_PrSur_bt_mean, color = "Baseline Scenario"), size = 0.6) +
  geom_line(aes(y = ES_PrSur_bt_q95, color = "Baseline Scenario"), size = 0.6, linetype = "dashed") +
  geom_line(aes(y = ES_PrSur_bt_mean.2, color = "Small EDA"), size = 0.6) +
  geom_line(aes(y = ES_PrSur_bt_q95.2, color = "Small EDA"), size = 0.6, linetype = "dashed") +
  scale_x_date(
    date_breaks = "3 years",
    date_labels = "%b\n%Y",
    limits = as.Date(c("2022-12-01", "2042-12-15"))
  ) +
  scale_y_continuous(
    limits = c(-0.05, 0.25),
    breaks = seq(-0.05, 0.25, 0.03)
  ) +
  ylab(NULL) +
  xlab(NULL) +
  labs(title = bquote(bold("Spain"))) +
  scale_color_manual(
    name = "Legend",
    values = c("Baseline Scenario" = "blue", "Small EDA" = "green"),
    labels = c("Baseline Scenario", "Small EDA")   
  ) +
  my_theme

ES_Surplus

#------------------------
####FRANCE:
FR_Surplus <- ggplot(df1, aes(x = date)) +
  geom_line(aes(y = FR_PrSur_bt_mean, color = "Baseline Scenario"), size = 0.6) +
  geom_line(aes(y = FR_PrSur_bt_q95, color = "Baseline Scenario"), size = 0.6, linetype = "dashed") +
  geom_line(aes(y = FR_PrSur_bt_mean.2, color = "Small EDA"), size = 0.6) +
  geom_line(aes(y = FR_PrSur_bt_q95.2, color = "Small EDA"), size = 0.6, linetype = "dashed") +
  scale_x_date(
    date_breaks = "3 years",
    date_labels = "%b\n%Y",
    limits = as.Date(c("2022-12-01", "2042-12-15"))
  ) +
  scale_y_continuous(
    limits = c(-0.05, 0.25),
    breaks = seq(-0.05, 0.25, 0.03)
  ) +
  ylab(NULL) +
  xlab(NULL) +
  labs(title = bquote(bold("France"))) +
  scale_color_manual(
    name = "Legend",
    values = c("Baseline Scenario" = "blue", "Small EDA" = "green"),
    labels = c("Baseline Scenario", "Small EDA")   
  ) +
  my_theme

FR_Surplus


#------------------------
####GREECE:
GR_Surplus <- ggplot(df1, aes(x = date)) +
  geom_line(aes(y = GR_PrSur_bt_mean, color = "Baseline Scenario"), size = 0.6) +
  geom_line(aes(y = GR_PrSur_bt_q95, color = "Baseline Scenario"), size = 0.6, linetype = "dashed") +
  geom_line(aes(y = GR_PrSur_bt_mean.2, color = "Small EDA"), size = 0.6) +
  geom_line(aes(y = GR_PrSur_bt_q95.2, color = "Small EDA"), size = 0.6, linetype = "dashed") +
  scale_x_date(
    date_breaks = "3 years",
    date_labels = "%b\n%Y",
    limits = as.Date(c("2022-12-01", "2042-12-15"))
  ) +
  scale_y_continuous(
    limits = c(-0, 0.40),
    breaks = seq(-0, 0.40, 0.03)
  ) +
  ylab(NULL) +
  xlab(NULL) +
  labs(title = bquote(bold("Greece"))) +
  scale_color_manual(
    name = "Legend",
    values = c("Baseline Scenario" = "blue", "Small EDA" = "green"),
    labels = c("Baseline Scenario", "Small EDA")   
  ) +
  my_theme

GR_Surplus


# plot(Data_S0_sigma$GR_PrSur_bt_mean, main = "Greece", type = "l", 
#      col = "blue",ylim = c(-0.07, 0.50),lwd = 1.5, cex=0.7)
# lines(Data_S0_sigma$GR_PrSur_bt_q95, col="blue",lty=2, lwd = 1.5)
# lines(Data_S1_sigma$GR_PrSur_bt_mean, col = "green",lwd = 1.5)
# lines(Data_S1_sigma$GR_PrSur_bt_q95, col = "green", lty=2, lwd = 1.5)
# 
# addLegend("bottomright",legend.names = c( "Baseline Scenario","Small EDA"),
#           col=c("blue","green"), lty=c(1,2), lwd=2, text.font=3)

#------------------------
####PORTUGAL:

PT_Surplus <- ggplot(df1, aes(x = date)) +
  geom_line(aes(y = PT_PrSur_bt_mean, color = "Baseline Scenario"), size = 0.6) +
  geom_line(aes(y = PT_PrSur_bt_q95, color = "Baseline Scenario"), size = 0.6, linetype = "dashed") +
  geom_line(aes(y = PT_PrSur_bt_mean.2, color = "Small EDA"), size = 0.6) +
  geom_line(aes(y = PT_PrSur_bt_q95.2, color = "Small EDA"), size = 0.6, linetype = "dashed") +
  scale_x_date(
    date_breaks = "3 years",
    date_labels = "%b\n%Y",
    limits = as.Date(c("2022-12-01", "2042-12-15"))
  ) +
  scale_y_continuous(
    limits = c(-0.05, 0.25),
    breaks = seq(-0.05, 0.25, 0.03)
  ) +
  ylab(NULL) +
  xlab(NULL) +
  labs(title = bquote(bold("Portugal"))) +
  scale_color_manual(
    name = "Legend",
    values = c("Baseline Scenario" = "blue", "Small EDA" = "green"),
    labels = c("Baseline Scenario", "Small EDA")   
  ) +
  my_theme

PT_Surplus


grid.arrange(DE_Surplus, ES_Surplus, FR_Surplus,
             GR_Surplus, IT_Surplus, PT_Surplus, ncol = 2)

dev.copy2pdf(width = 6.5, height=9, out.type = "pdf",file="figures/Figure_Surplus.eps")
dev.off()


#-------------------------------------------------------------------------------
### EDA:  
#-------------------------------------------------------------------------------
#------------------------
#SMALL EDA
#------------------------
plot(Stochastic_EDA$R_EDA_1_mean,main = "EDA Assets and Liabilities", 
     type = "l", col = "green", lwd =2, ylim = c(-0.02, 0.30), cex=0.7)
lines(Stochastic_EDA$L_EDA_1_mean, col="green", lwd =2, lty=2)
lines(Stochastic_EDA$B_EDA_1_mean, col="red",lwd =2)
lines(Stochastic_EDA$SC_EDA_1_mean,
      col = "red", lty=2, lwd =2)
addLegend("topleft",
          legend.names = c( "Reserves","Loans", "Bonds", "Solvency Capital"),
          col=c("green", "green", "red", "red"), lty=c(1,2,1,2,2), lwd=2, text.font=3)

# dev.copy2pdf(width = 8, out.type = "pdf",file="figures/Figure_6.pdf")
# dev.off()




#-------------------------------------------------------------------------------
####Count of Fiscal Savings 

#------------------------
#SMALL EDA
#------------------------
k=1
PV_savings <- as.data.frame(vector(mode ="numeric", length = 6))
TV_savings <- as.data.frame(vector(mode ="numeric", length = 6))

for (Name in MSList[MSList$Simulation=="Y",4])
{
  PrSur_diff <- Data_S0[,paste(Name, "PrSur_bt_mean", sep="_")] - Data_S1[,paste(Name, "PrSur_bt_mean", sep="_")]
  colnames(PrSur_diff)<- "Diff"
  
  DF <- as.data.frame(vector(mode ="numeric", length = 21))
  DF[1,] <- 1/(1+Data_S1[1,paste(Name, "IC_mean", sep="_")][[1]])
  
  for (i in 2:21) {
    DF[i,]<- DF[i-1,]*(1/(1+Data_S1[i,paste(Name, "IC_mean", sep="_")][[1]]))}
  
  temp1 <- as.data.frame(vector(mode ="numeric", length = 21))
  temp2 <- as.data.frame(vector(mode ="numeric", length = 21))
  for (i in 1:21) {
    temp1[i,] <- PrSur_diff[i,][[1]]
    temp2[i,] <- PrSur_diff[i,][[1]]*DF[i,]}
  
  TV_savings[k,] <- sum(temp1[c(1:21),1])
  PV_savings[k,] <- sum(temp2[c(1:21),1])
  k= k+1
}

PV_savings <- cbind(MSList[MSList$Simulation=="Y",4],PV_savings)
TV_savings <- cbind(MSList[MSList$Simulation=="Y",4],TV_savings)

Savings_Small_EDA <- cbind(PV_savings,TV_savings[,-1])
colnames(Savings_Small_EDA) <- c("MS","Present Value FS", "Total Value FS")

rm(PrSur_diff, PV_savings, TV_savings)


