# clear memory 
rm(list=ls())
#Check the working directory before importing else provide full path
#setwd(path)
setwd(dirname(rstudioapi::getActiveDocumentContext()$path))
# packages used


listofpackages <- c("dygraphs", "dplyr","ellipse","reshape2","ggplot2","xts","readxl","readxl","stats",
                    "plotly", "hrbrthemes", "zoo", "dynlm", "gridExtra")

for (j in listofpackages){
  if(sum(installed.packages()[, 1] == j) == 0) {
    install.packages(j)
  }
  library(j, character.only = T)
}

##### DATABASE IN PANEL FORMAT: 
Sigma_Debt <- data_frame <- read_xlsx("data/Sigma_Debt.xlsx")
Sigma_Debt_1 <- data_frame <- read_xlsx("data/Sigma_Debt_1.xlsx")
Spread_Variance <- data_frame <- read_xlsx("data/Spread_Variance.xlsx")
Spread_Variance_1 <- data_frame <- read_xlsx("data/Spread_Variance_1.xlsx")
Spread_Variance_2 <- data_frame <- read_xlsx("data/Spread_Variance_2.xlsx")


#-------------------------------------------------------------------------------
## Graph Sigma Residual and Debt to GDP (mean)
Panel_left = ggplot(data = Sigma_Debt, aes(x = d, y = sigma)) +
  geom_point() +
  geom_smooth(method = "lm", se = FALSE, color = "red", size = 1) +
  geom_text(aes(label = MS), nudge_x = 0, nudge_y = 0.005, size = 3, color = "black") +
  labs(x = "Debt to GDP", y = "Residuals s.t. Deviation") +
  ggtitle("Volatility and Debt to GDP") +
  theme_minimal()

## Graph Sigma Residual and Debt to GDP (mean), with Germany
Panel_right = ggplot(data = Sigma_Debt_1, aes(x = d, y = sigma)) +
  geom_point() +
  geom_smooth(method = "lm", se = FALSE, color = "red", size = 1) +
  geom_text(aes(label = MS), nudge_x = 0, nudge_y = 0.005, size = 3, color = "black") +
  labs(x = "Debt to GDP", y = "Residuals s.t. Deviation") +
  ggtitle("Volatility and Debt to GDP") +
  theme_minimal()

grid.arrange(Panel_left, Panel_right, ncol = 1)

## Regression Sigma residual and Debt to GDP (mean)
regression_model <- lm(sigma ~ d - 1, data = Sigma_Debt)
summary(regression_model)

## Regression Sigma residual and Debt to GDP (mean), with Germany
regression_model <- lm(sigma ~ d - 1, data = Sigma_Debt_1)
summary(regression_model)



#-------------------------------------------------------------------------------
## Graph Sigma Spread and Debt to GDP
ggplot(data = Spread_Variance, aes(x = D, y = Sigma)) +
  geom_point() +
  geom_smooth(method = "lm", se = FALSE, color = "red") +
  labs(x = "Debt to GDP", y = "Spread s.t. Deviation") +
  ggtitle("Volatility and Debt to GDP") +
  theme_minimal()

## Regression Sigma Spread and Debt to GDP (mean)
regression_model <- lm(Sigma ~ D - 1, data = Spread_Variance)
summary(regression_model)


## Graph Sigma Spread and Debt to GDP, non linear

formula <- Sigma ~ b*(D^2)
non_linear_model <- nls(formula, data = Spread_Variance, start = list(b = 0.02))
summary(non_linear_model)

Spread_Variance$Sigma_pred <- predict(non_linear_model)
ggplot(data = Spread_Variance, aes(x = D, y = Sigma)) +
  geom_point() +
  geom_line(aes(y = Sigma_pred), color = "blue", size = 1) +
  labs(x = "Debt to GDP", y = "Spread s.t. Deviation") +
  ggtitle("Volatility and Debt to GDP") +
  theme_minimal()


# Only Countries in Small EDA
#-------------------------------------------------------------------------------
# Coefficiente angolare e intercetta della seconda retta personalizzata
slope_custom <- 0.019278  # Sostituisci con il coefficiente angolare desiderato per la seconda retta
intercept_custom <- 0  # Sostituisci con l'intercetta desiderata per la seconda retta

# Grafico scatter plot con entrambe le rette
ggplot(data = Spread_Variance_1, aes(x = D, y = Sigma)) +
  geom_point() +
  geom_smooth(method = "lm", formula = y ~ x - 1, se = FALSE, color = "red") +
  geom_abline(intercept = intercept_custom, slope = slope_custom, color = "green", linetype = "dotted", size = 1) +
  labs(x = "Debt to GDP", y = "Spread s.t. Deviation") +
  ggtitle("Volatility and Debt to GDP") +
  theme_minimal()

## Regression Sigma Spread and Debt to GDP (mean)
regression_model <- lm(Sigma ~ D - 1, data = Spread_Variance_1)
summary(regression_model)

## Graph Sigma Spread and Debt to GDP, non linear
formula <- Sigma ~ b*(D^2)
non_linear_model <- nls(formula, data = Spread_Variance_1, start = list(b = 0.02))
summary(non_linear_model)

Spread_Variance_1$Sigma_pred <- predict(non_linear_model)
ggplot(data = Spread_Variance_1, aes(x = D, y = Sigma)) +
  geom_point() +
  geom_line(aes(y = Sigma_pred), color = "blue", size = 1) +
  labs(x = "Debt to GDP", y = "Spread s.t. Deviation") +
  ggtitle("Volatility and Debt to GDP") +
  theme_minimal()

# Only Countries in Small EDA, without Germany
#-------------------------------------------------------------------------------
# Coefficiente angolare e intercetta della seconda retta personalizzata
slope_custom <- 0.019278  # Sostituisci con il coefficiente angolare desiderato per la seconda retta
intercept_custom <- 0  # Sostituisci con l'intercetta desiderata per la seconda retta

# Grafico scatter plot con entrambe le rette
ggplot(data = Spread_Variance_2, aes(x = D, y = Sigma)) +
  geom_point() +
  geom_smooth(method = "lm", formula = y ~ x - 1, se = FALSE, color = "red") +
  geom_abline(intercept = intercept_custom, slope = slope_custom, color = "green", linetype = "dotted", size = 1) +
  labs(x = "Debt to GDP", y = "Spread s.t. Deviation") +
  ggtitle("Volatility and Debt to GDP") +
  theme_minimal()

## Regression Sigma Spread and Debt to GDP (mean)
regression_model <- lm(Sigma ~ D - 1, data = Spread_Variance_2)
summary(regression_model)

## Graph Sigma Spread and Debt to GDP, non linear
formula <- Sigma ~ b*(D^2)
non_linear_model <- nls(formula, data = Spread_Variance_2, start = list(b = 0.02))
summary(non_linear_model)

Spread_Variance_2$Sigma_pred <- predict(non_linear_model)
ggplot(data = Spread_Variance_2, aes(x = D, y = Sigma)) +
  geom_point() +
  geom_line(aes(y = Sigma_pred), color = "blue", size = 1) +
  labs(x = "Debt to GDP", y = "Spread s.t. Deviation") +
  ggtitle("Volatility and Debt to GDP") +
  theme_minimal()


