# clear memory 
rm(list=ls())
#Check the working directory before importing else provide full path
#setwd(path)
setwd(dirname(rstudioapi::getActiveDocumentContext()$path))
# packages used


listofpackages <- c("dygraphs", "systemfit", "stargazer", "texreg", "dplyr","ellipse","reshape2","ggplot2","xts","readxl","stats",
                    "plotly", "hrbrthemes", "zoo", "dynlm")

for (j in listofpackages){
  if(sum(installed.packages()[, 1] == j) == 0) {
    install.packages(j)
  }
  library(j, character.only = T)
}


##### N.B. Deficit IN THE DATA  is (t-g)

##### DATABASE IN PANEL FORMAT: 

MSList <- read.table("data/MSList2.csv",header = T,sep = ";",dec = ".", stringsAsFactors = FALSE)
DB <- read_excel("data/DB.xlsx")
RecYears <- read_excel("data/RecessionYears.xlsx")
SwapMonthlyData <- read.table("data/SwapMonthlyData.csv",header = T,sep = ";",dec = ".", stringsAsFactors = FALSE) 
SwapMonthlyData1 <- read.table("data/SwapMonthlyData1.csv",header = T,sep = ";",dec = ".", stringsAsFactors = FALSE)
TotalDataBYMS <- read.table("data/TotalDataByMS_2022.csv",header = T,sep = ";",dec = ".", stringsAsFactors = FALSE) 
TTC <- read.table("data/TTC SOV.csv",header = F,sep = ";",dec = ".")
MES <- read_excel("data/MES.xlsx")

## -----------------------------------------------------------------------------
## Get the data in .xts format for annual time series 
## -----------------------------------------------------------------------------

dates <-seq(as.Date("1995-12-15"),length=28, by="years")
dates2<-seq(as.Date("2000-12-15"),length=23, by="years")
dates3<-seq(as.Date("2021-12-15"),length=30, by="years")
dates4<- seq(as.Date("2019-12-15"),length=32, by="years")
dates5<- seq(as.Date("2001-01-15"),length=264, by="months")
dates6 <-seq(as.Date("2001-01-15"),length=264, by="months")
dates7 <-seq(as.Date("2022-12-15"),length=21, by="years")

## -----------------------------------------------------------------------------
## Databanking
## -----------------------------------------------------------------------------

###interpolating missing month 10Y Greece (07/2015)
k <-as.numeric(which(is.na(TotalDataBYMS$Yield)))
TotalDataBYMS[k,6] <- (TotalDataBYMS[k-1,6]+TotalDataBYMS[k+1,6])/2
###--------------------------------------------------

TestFirst <- 0
for (MS in MSList[MSList$Exclude=="N",1])
{
  DB_ByMS <-DB[DB$MS==MS,]
  DB_ByMS<-as.data.frame(DB_ByMS[, -c(1,2)])
  tsdata <- as.xts(DB_ByMS, order.by = dates)
  
  Rates_ByMS <- TotalDataBYMS[TotalDataBYMS$MS==MS,]
  Rates_ByMS<-as.data.frame(Rates_ByMS[,-c(1:5,7,9)])
  tsdata1 <- as.xts(Rates_ByMS, order.by=dates6)
  tsdata.yearly1 = to.yearly(tsdata1[, 1])[,4]
  tsdata.yearly2 = to.yearly(tsdata1[, 2])[,4]
  
  tsdata = cbind(tsdata,tsdata.yearly1,tsdata.yearly2)
  
  names = c("GDP", "Debt", "PrimarySurplus", "Surplus", "SFA", "Growth","10Y","IC")
  NameByMS <-MSList[MSList$MS==MS,4]
  j=1
  for (i in names) {
    names[j] = paste(NameByMS,i, sep="_")
    j=j+1 
  }
  colnames(tsdata)=names
  if (TestFirst==0) 
  {
    MyData <- tsdata
    TestFirst <- 1
    
  } else {
    MyData <- cbind(MyData, tsdata)
  } 
}
rm(tsdata,tsdata.yearly1, tsdata.yearly2, tsdata1, DB_ByMS, Rates_ByMS)


SwapMonthlyData<-as.data.frame(SwapMonthlyData[,-c(1:2)])
SwapMonthly <- as.xts(SwapMonthlyData, order.by=dates5)
SwapMonthlyData1<-as.data.frame(SwapMonthlyData1[,-c(1:2)])
SwapMonthly1 <- as.xts(SwapMonthlyData1, order.by=dates5)
ts.year_IR10 <- to.yearly(SwapMonthly[, 1])[,4]
ts.year_IR1 <- to.yearly(SwapMonthly1[, 1])[,4]
# tsdata.yearly_IR = to.yearly(SwapMonthly[, 1])[,4]
colnames(ts.year_IR10) <- c("IR10")
colnames(ts.year_IR1) <- c("IR1")
IR_1Y<- ts.year_IR1
IR_10Y<- ts.year_IR10


rm(ts.year_IR10,ts.year_IR1, SwapMonthlyData,SwapMonthlyData1,SwapMonthly,SwapMonthly1)
## -----------------------------------------------------------------------------
## Calculate the Average Cost of Debt: 
## -----------------------------------------------------------------------------

TestFirst <- 0
for (Name in MSList[MSList$Exclude=="N",4])
{
  tsdata<- as.xts(array(data = NA, dim = 28), order.by=dates)
  t1<-nrow(tsdata)
  colnames(tsdata)<- c("CM")
  for (i in 2:t1) {
    tsdata[i, "CM"][[1]]= (- MyData[i,paste(Name, "Surplus", sep="_")][[1]] + 
                             + MyData[i,paste(Name, "PrimarySurplus", sep="_")][[1]]) /MyData[i-1,paste(Name, "Debt", sep="_")][[1]]
  }
  names = c("CM")
  j=1
  for (i in names) {
    names[j] = paste(Name,i, sep="_")
    j=j+1 
  }
  colnames(tsdata)=names
  if (TestFirst==0) 
  {
    AvgCost <- tsdata
    TestFirst <- 1
    
  } else {
    AvgCost <- cbind(AvgCost, tsdata)
  } 
}
rm(tsdata)


MyData<- cbind(MyData, AvgCost)
MyData <- MyData[,sort(names(MyData))]

MyData <- cbind(MyData, IR_1Y,IR_10Y)

#-----------------------------------
# generating SFA for Ireland

SFA<- as.xts(array(data = NA, dim = 28), order.by=dates)
t1<-nrow(SFA)
Name <- "IE"
for (i in 2:t1) {
  SFA[i,][[1]]= MyData[i,paste(Name, "Surplus", sep="_")][[1]] + 
    + (MyData[i,paste(Name, "Debt", sep="_")][[1]] - MyData[i-1,paste(Name, "Debt", sep="_")][[1]])
}

MyData$IE_SFA<- SFA

rm(SFA)


## -----------------------------------------------------------------------------
## Simulation - Within Sample
## -----------------------------------------------------------------------------


TestFirst <- 0
for (Name in MSList[MSList$Exclude=="N",4])
{
  tsdata<- as.xts(array(data = NA, dim = 23), order.by=dates2)
  t1<-nrow(tsdata)
  colnames(tsdata)<- c("B")
  tsdata[1, c("B")] <- MyData[6,paste(Name, "Debt", sep="_")][[1]]
  tsdata$B_Cov <- 0
  tsdata$B_Rec <- 0
  tsdata$B_Slow <- 0
  tsdata$B_Fast <- 0
  tsdata$bCov<- 0
  tsdata$bS<- 0
  tsdata$bF<- 0
  tsdata$b<- 0 
  dummy <- ifelse((MyData[,paste(Name, "PrimarySurplus", sep="_")]+MyData[,paste(Name, "SFA", sep="_")]) < 0, 1, 0)
  
  for (i in 2:t1) {
    tsdata[i, "B"][[1]] = tsdata[i-1, "B"][[1]]*(1 + MyData[5+i,paste(Name, "CM", sep="_")][[1]])+
      - MyData[5+i,paste(Name, "PrimarySurplus", sep="_")][[1]] + MyData[5+i,paste(Name, "SFA", sep="_")][[1]]
    
    tsdata[i, "B_Cov"][[1]] = tsdata[i-1, "B_Cov"][[1]] +
      + dummy[5+i,]*RecYears[5+i,3][[1]]*(-MyData[5+i,paste(Name, "PrimarySurplus", sep="_")][[1]]+ 
                                            + MyData[5+i,paste(Name, "SFA", sep="_")][[1]]+
                                            + tsdata[i-1, "B"][[1]]*MyData[5+i,paste(Name, "CM", sep="_")][[1]])
    
    tsdata[i, "B_Rec"][[1]] = tsdata[i-1, "B_Rec"][[1]] +
      + dummy[5+i,]*RecYears[5+i,2][[1]]*(-MyData[5+i,paste(Name, "PrimarySurplus", sep="_")][[1]]+ 
                                            + MyData[5+i,paste(Name, "SFA", sep="_")][[1]]+
                                            + tsdata[i-1, "B"][[1]]*MyData[5+i,paste(Name, "CM", sep="_")][[1]])
    
    tsdata[i, "B_Slow"][[1]] =  tsdata[i, "B_Cov"][[1]] + tsdata[i, "B_Rec"][[1]]
    
    tsdata[i, "B_Fast"][[1]] = tsdata[i, "B"][[1]]- tsdata[i, "B_Slow"][[1]] 
    
    tsdata[i, "bCov"][[1]] = tsdata[i, "B_Cov"][[1]]/MyData[5+i,paste(Name, "GDP", sep="_")][[1]]
    
    tsdata[i, "bS"][[1]] = tsdata[i, "B_Slow"][[1]]/MyData[5+i,paste(Name, "GDP", sep="_")][[1]]
    
    tsdata[i, "bF"][[1]] = tsdata[i, "B_Fast"][[1]]/MyData[5+i,paste(Name, "GDP", sep="_")][[1]]
    
    tsdata[i, "b"][[1]] = tsdata[i, "B"][[1]]/MyData[5+i,paste(Name, "GDP", sep="_")][[1]]
    
  }
  names = c("B", "B_Cov","B_Rec", "B_Slow", "B_Fast", "bCov", 
            "bS", "bF", "b")
  j=1
  for (i in names) {
    names[j] = paste(Name,i, sep="_")
    j=j+1 
  }
  colnames(tsdata)=names
  if (TestFirst==0) 
  {
    SimDebt1 <- tsdata
    TestFirst <- 1
    
  } else {
    SimDebt1 <- cbind(SimDebt1, tsdata)
  } 
}
rm(tsdata, dummy )

## -----------------------------------------------------------------------------
## Estimation 
## -----------------------------------------------------------------------------

MyData_subset <- MyData["2001-12-15/2022-12-15"]
###----------------------
### data transformation  
###----------------------
name <-c("DE","ES","FR","GR","IT","PT")

MyData_subset$AGG_GDP<-MyData_subset$DE_GDP+MyData_subset$FR_GDP+MyData_subset$IT_GDP+
  +MyData_subset$ES_GDP+MyData_subset$PT_GDP+MyData_subset$GR_GDP
MyData_subset$DE_W<-MyData_subset$DE_GDP/MyData_subset$AGG_GDP
MyData_subset$FR_W<-MyData_subset$FR_GDP/MyData_subset$AGG_GDP
MyData_subset$IT_W<-MyData_subset$IT_GDP/MyData_subset$AGG_GDP
MyData_subset$ES_W<-MyData_subset$ES_GDP/MyData_subset$AGG_GDP
MyData_subset$PT_W<-MyData_subset$PT_GDP/MyData_subset$AGG_GDP
MyData_subset$GR_W<-MyData_subset$GR_GDP/MyData_subset$AGG_GDP
MyData_subset$AGG_Gr<- stats::lag(MyData_subset$DE_W)*MyData_subset$DE_Growth+
  +stats::lag(MyData_subset$FR_W)*MyData_subset$FR_Growth+
  +stats::lag(MyData_subset$IT_W)*MyData_subset$IT_Growth+
  +stats::lag(MyData_subset$ES_W)*MyData_subset$ES_Growth+
  +stats::lag(MyData_subset$PT_W)*MyData_subset$PT_Growth+
  +stats::lag(MyData_subset$GR_W)*MyData_subset$GR_Growth

for (Name in MSList[MSList$Simulation=="Y",4])
{
  temp1 <- stats::lag(MyData_subset[,paste(Name, "Growth", sep="_")])
  colnames(temp1)<- paste(Name, "Growth_lag", sep="_")
  
  temp2 <- stats::lag(MyData_subset[,paste(Name, "CM", sep="_")])
  colnames(temp2)<- paste(Name, "CM_lag", sep="_")
  
  temp3 <- MyData_subset[,paste(Name, "10Y", sep="_")]- MyData_subset$DE_10Y
  colnames(temp3)<- paste(Name, "spread", sep="_")
  temp4 <- stats::lag(temp3)
  colnames(temp4)<- paste(Name, "spread_lag", sep="_")
  
  temp5 <- MyData_subset[,paste(Name, "10Y", sep="_")] - MyData_subset$IR10 
  colnames(temp5)<- paste(Name, "spread_swap", sep="_")
  temp6 <- stats::lag(temp5)
  colnames(temp6)<- paste(Name, "spread_swap_lag", sep="_")
  
  temp7 <- MyData_subset[,paste(Name, "PrimarySurplus", sep="_")]/MyData_subset[,paste(Name, "GDP", sep="_")]
  colnames(temp7)<- paste(Name, "PrSur", sep="_")
  temp8 <- stats::lag(temp7)
  colnames(temp8)<- paste(Name, "PrSur_lag", sep="_")
  temp9 <- MyData_subset[,paste(Name, "Debt", sep="_")]/MyData_subset[,paste(Name, "GDP", sep="_")]
  colnames(temp9)<- paste(Name, "b", sep="_")
  temp10 <- stats::lag(temp9)
  colnames(temp10)<- paste(Name, "b_lag", sep="_")
  
  MyData_subset <- cbind(MyData_subset,temp1,temp2,temp3,temp4,temp5,temp6,temp7,temp8,temp9,temp10)
}

MyData_subset$IR1_lag<-stats::lag(MyData_subset$IR1)

rm(temp1,temp2,temp3,temp4,temp5,temp6,temp7,temp8,temp9,temp10)

###----------------------
###Graphical Exploration
###----------------------
plot(MyData_subset$IT_Growth, main = "Growth - Italy",
     type = "l", col = "black",ylim = c(-0.08, 0.10), cex=0.7)
lines(MyData_subset$IT_spread,col = "blue",lwd=2, lty=5)
lines(MyData_subset$IR1, col="orange",lwd=2, lty=5)
lines(MyData_subset$IT_PrSur,col = "red",lwd=2, lty=5)

addLegend("bottomleft",
          legend.names = c( "Growth", "BTP-BUND spread","Primary Surplus","Monetary Policy"),
          col=c("black","blue","red", "orange"), cex=0.9, lwd=2, text.font=3)


###----------------------
###System estimation for GDP growth
###----------------------
eqDE_gr <- DE_Growth ~  IR1_lag  + DE_PrSur_lag
eqES_gr <- ES_Growth ~  IR1_lag + ES_spread_lag + ES_PrSur_lag
eqFR_gr <- FR_Growth ~  IR1_lag + FR_spread_lag + FR_PrSur_lag
eqGR_gr <- GR_Growth ~  IR1_lag + GR_spread_lag + GR_PrSur_lag
eqIT_gr <- IT_Growth ~  IR1_lag + IT_spread_lag + IT_PrSur_lag
eqPT_gr <- PT_Growth ~  IR1_lag + PT_spread_lag + PT_PrSur_lag
system_Gr <- list(eqDE_gr,eqES_gr, eqFR_gr,  eqGR_gr, eqIT_gr, eqPT_gr)
#inst <- ~ income + farmPrice + trend
#unrestricted estimation by specified method 
sys_gr_unr <- systemfit( system_Gr, "SUR", data = MyData_subset )
#restricted estimation by specified method

restrictM <- matrix(c(0,1,0,0,-1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 
                      0,1,0,0,0,0,0,0,-1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                      0,1,0,0,0,0,0,0,0,0,0,0,-1,0,0,0,0,0,0,0,0,0,0,
                      0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-1,0,0,0,0,0,0,
                      0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-1,0,0, 
                      0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 
                      0,0,1,0,0,0,0,0,0,0,-1,0,0,0,0,0,0,0,0,0,0,0,0,
                      0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,
                      0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-1,0,0,0,0,
                      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1),
                    byrow=TRUE, nrow=10, ncol=23)

sys_gr_rest <-systemfit(system_Gr, method="SUR", data = MyData_subset,
                        restrict.matrix = restrictM)
Resid_G <- residuals(sys_gr_rest)
Resid_G <- Resid_G[-1,]
colnames(Resid_G)<- c("DE_G","ES_G","FR_G", "GR_G", "IT_G","PT_G")

Coeff_G<-matrix(0,4,6)
Coeff_G[1:2,1]<- coef(sys_gr_rest)[1:2]
Coeff_G[3,1]<- 0
Coeff_G[4,1]<- coef(sys_gr_rest)[3]
Coeff_G[,2]<- coef(sys_gr_rest)[4:7]
Coeff_G[,3]<- coef(sys_gr_rest)[8:11]
Coeff_G[,4]<- coef(sys_gr_rest)[12:15]
Coeff_G[,5]<- coef(sys_gr_rest)[16:19]
Coeff_G[,6]<- coef(sys_gr_rest)[20:23]

colnames(Coeff_G)<- c("DE","ES","FR","GR","IT","PT")

fit_gr_rest <- fitted(sys_gr_rest)

#dates8<-seq(as.Date("2001-12-15"),length=22, by="years")
#temp <- as.xts(fit_gr_rest$eq3, order.by=dates8)
#plot(temp)

###----------------------
###System estimation for average cost 
###----------------------
eqDE_cm <- DE_CM ~  DE_CM_lag + IR1+ DE_10Y -1
eqES_cm <- ES_CM ~  ES_CM_lag + IR1+ ES_10Y -1
eqFR_cm <- FR_CM ~  FR_CM_lag + IR1+ FR_10Y -1
eqGR_cm <- GR_CM ~  GR_CM_lag + IR1+ GR_10Y -1
eqIT_cm <- IT_CM ~  IT_CM_lag + IR1+ IT_10Y -1
eqPT_cm <- PT_CM ~  PT_CM_lag + IR1+ PT_10Y -1

system_cm <- list( eqDE_cm, eqES_cm, eqFR_cm, eqGR_cm, eqIT_cm, eqPT_cm )
#inst <- ~ income + farmPrice + trend
#unrestricted estimation by specified method 
sys_cm_unr <- systemfit( system_cm, "SUR", data = MyData_subset )

fit_cm_unr <- fitted(sys_cm_unr)
#dates8<-seq(as.Date("2001-12-15"),length=22, by="years")
#temp <- as.xts(fit_cm_unr$eq6, order.by=dates8)
#plot(temp)
#lines(MyData_subset$GR_CM)

Resid_CM <- residuals(sys_cm_unr)
Resid_CM <- Resid_CM[-1,]
colnames(Resid_CM)<- c("DE_CM","ES_CM","FR_CM", "GR_CM","IT_CM","PT_CM")

Coeff_CM<-matrix(0,3,6)
Coeff_CM[,1]<- coef(sys_cm_unr)[1:3]
Coeff_CM[,2]<- coef(sys_cm_unr)[4:6]
Coeff_CM[,3]<- coef(sys_cm_unr)[7:9]
Coeff_CM[,4]<- coef(sys_cm_unr)[10:12]
Coeff_CM[,5]<- coef(sys_cm_unr)[13:15]
Coeff_CM[,6]<- coef(sys_cm_unr)[16:18]

colnames(Coeff_CM)<- c("DE","ES","FR","GR","IT","PT")


###----------------------
### 1-year Swap rate regression 
###----------------------
regr_1Y<- lm(MyData_subset$IR1 ~ stats::lag(MyData_subset$IR1) + MyData_subset$AGG_Gr)

Resid_1Y <- as.data.frame(regr_1Y[["residuals"]][1:21])
colnames(Resid_1Y)<- c("IR1")
Coeff_1Y <- as.data.frame(regr_1Y$coefficients)
colnames(Coeff_1Y)<- c("IR1")

# regr_1Y_fit<-as.data.frame(as.numeric(regr_1Y$fitted.values))
# temp1 <- as.xts(regr_1Y_fit, order.by=dates8)
# plot(temp)

###----------------------
### 10 Swap rate (EDA rate) regression
###----------------------
regr_10Y<- lm(MyData_subset$IR10 ~ stats::lag(MyData_subset$IR10) + MyData_subset$IR1)

Resid_10Y <- as.data.frame(regr_10Y[["residuals"]][1:21])
colnames(Resid_10Y)<- c("IR10")
Coeff_10Y <- as.data.frame(regr_10Y$coefficients)
colnames(Coeff_10Y)<- c("IR10") 

###----------------------
### Spread regressions
###----------------------

### first dynamics of the 10-Y bund 
eqDE_10Y<-lm(MyData_subset$DE_10Y ~ stats::lag(MyData_subset$DE_10Y) + MyData_subset$IR1)
Resid_DE<- as.data.frame(eqDE_10Y[["residuals"]][1:21])
colnames(Resid_DE)<- c("DE")
Coeff_DE <- as.data.frame(eqDE_10Y$coefficients)
colnames(Coeff_DE)<- c("DE") 

### then a system for spreads on bunds 

eqES_sp <- ES_spread ~  1 + ES_b_lag 
eqFR_sp <- FR_spread ~  1 + FR_b_lag
eqGR_sp <- GR_spread~  1 + GR_b_lag
eqIT_sp <- IT_spread~  1 +IT_b_lag
eqPT_sp <- PT_spread ~  1 + PT_b_lag

system_sp <- list(eqES_sp, eqFR_sp, eqGR_sp, eqIT_sp, eqPT_sp )
#inst <- ~ income + farmPrice + trend
#unrestricted estimation by specified method 
sys_sp_unr <- systemfit( system_sp, "SUR", data = MyData_subset )

restrictM_sp <- matrix(c(0,1,0,-1,0,0,0,0,0,0, 
                         0,1,0,0,0,-1,0,0,0,0,
                         0,1,0,0,0,0,0,-1,0,0,
                         0,1,0,0,0,0,0,0,0,-1),
                       byrow=TRUE, nrow=4, ncol=10)


sys_sp_rest <-systemfit(system_sp, method="SUR", data = MyData_subset,
                        restrict.matrix = restrictM_sp)
Resid_spread <- residuals(sys_sp_rest)
Resid_spread<- Resid_spread[-1,]
colnames(Resid_spread)<- c("ES_sp","FR_sp","GR_sp","IT_sp","PT_sp")
write.table(Resid_spread, file = "data/Resid_spread.csv", row.names = F, sep = ";", dec='.')


Coeff_spread<-matrix(0,2,5)
Coeff_spread[,1]<- coef(sys_sp_rest)[1:2]
Coeff_spread[,2]<- coef(sys_sp_rest)[3:4]
Coeff_spread[,3]<- coef(sys_sp_rest)[5:6]
Coeff_spread[,4]<- coef(sys_sp_rest)[7:8]
Coeff_spread[,5]<- coef(sys_sp_rest)[9:10]
colnames(Coeff_spread)<- c("ES","FR","GR","IT","PT")

fit_sp_rest <- fitted(sys_sp_rest)
# dates8<-seq(as.Date("2001-12-15"),length=22, by="years")
# temp <- as.xts(fit_sp_rest$eq3, order.by=dates8)
# plot(MyData_subset$IT_spread)
# lines(temp, col="red")

# stargazer(regr_1Y, regr_10Y,  title="Regression Results",
#           dep.var.labels=c("swap 1 year", "swap 10Y"),
#           # covariate.labels=c("IR swap 1 year at (t-1)","Eurozone Growth","Constant"),
#           no.space=TRUE)
# 
# texreg(list(sys_cm_unr))

## -----------------------------------------------------------------------------
## Stochastic simulation - Out of Sample - Exogenous variables and debt cost 
## -----------------------------------------------------------------------------

#input to simulate idiomatic cost, Through the Cycle Transition matrix, TTC)
TTC <- as.matrix(TTC) #ncol=8, nrow=8)
D_TTC<-diag(eigen(TTC)$values)
S_TTC<-eigen(TTC)$vectors
S_TTC_Inv<-solve(S_TTC)

UnitMatrix <- diag(1,8,8)
unitVector<- as.vector(rep.int(1, 8))
v <- as.vector(c(0,0,0,0,0,0,0,1))
rr <- .30

Grade <- c("AAA","AA", "A","BBB", "BB","B","CCC", "CC", "C","" )
Position <- c(1,2,3,4,5,6,7,7,7,7)
Ratings <- data.frame(Grade, Position)
Rating <- matrix(c("AAA","A","AA","BB","BBB","BBB"),1,6)
colnames(Rating)<- name
#SET HERE THE NUMBER OF REPLICATIONS 
nrep <- 1000

###BOOTSTRAPPING RESIDUALS: 

Names <- c("ES","FR","GR","IT","PT")

resid_matrix <- cbind(Resid_G, Resid_CM, Resid_1Y, Resid_10Y, Resid_DE, Resid_spread)

bt_matrix <- lapply(1:nrep, matrix, data= 0, nrow=21, ncol=20)


for (i in 1:nrep){
  bt_matrix[[i]] <- resid_matrix[sample(1:20, 21, replace=TRUE), ] }

res1_bt <- lapply(1:6, matrix, data= 0, nrow=21, ncol=nrep)
res4_bt <- lapply(1:6, matrix, data= 0, nrow=21, ncol=nrep)
res5_bt <- lapply(1:5, matrix, data= 0, nrow=21, ncol=nrep)
res5_EDA_bt <- lapply(1:5, matrix, data= 0, nrow=21, ncol=nrep)
res2_bt <- array(0, c(21, nrep))
res3_bt <- array(0, c(21, nrep))
res6_bt <- array(0, c(21, nrep))

names(res1_bt) <- name
names(res4_bt) <- name
names(res5_bt) <- Names
names(res5_EDA_bt) <- Names

for (i in 1:nrep){
  res2_bt[,i] <- bt_matrix[[i]][["IR1"]]
  res3_bt[,i] <- bt_matrix[[i]][["IR10"]]
  res6_bt[,i] <- bt_matrix[[i]][["DE"]]
  
  for (Name in MSList[MSList$Simulation=="Y",4])
  { res1_bt[[Name]][,i] <- bt_matrix[[i]][[paste(Name, "G", sep="_")]]
  res4_bt[[Name]][,i] <- bt_matrix[[i]][[paste(Name, "CM", sep="_")]]}
  for (Name in Names)
  { res5_bt[[Name]][,i] <- bt_matrix[[i]][[paste(Name, "sp", sep="_")]]}
   
}



sigma_MS = matrix(0,21,5)
colnames(sigma_MS)=Names
sigma_MS = as.data.frame(sigma_MS)
 
for (Name in Names){
  for (k in 1:21) {
    sigma_MS[k,Name] = sd(res5_bt[[Name]][k,])
  }
}

for (Name in Names){
  for (i in 1:nrep){
    for (k in 1:21){
      res5_EDA_bt[[Name]][k,i] = res5_bt[[Name]][k,i]/sigma_MS[k,Name]
    }
  }
}




###----------------------
### NO EDA SCENARIO (spread on 10Y swap in output growth)
###---------------------- 
k <- as.numeric(nrow(MyData_subset))
k1 <- as.numeric(nrow(MyData))
k2 <- as.numeric(nrow(SimDebt1))

###----------------------
### Coefficients
#b0<- Coeff_1Y[1,]
b0<- 0
b1<- Coeff_1Y[2,]
b2<- Coeff_1Y[3,]

c0<- Coeff_10Y[1,]
c1<- Coeff_10Y[2,]
c2<- Coeff_10Y[3,]

beta <- 0.05
gamma <- 0.02
delta <- 0.1

b <- 0.60

###----------------------  
### Variables

r_bt <- lapply(1:6, matrix, data= 0, nrow=21, ncol=nrep)
r_10Y_bt <- lapply(1:6, matrix, data= 0, nrow=21, ncol=nrep)
g_bt <- lapply(1:6, matrix, data= 0, nrow=21, ncol=nrep)
GDP <- lapply(1:6, matrix, data= 0, nrow=21, ncol=nrep)
W <- lapply(1:6, matrix, data= 0, nrow=21, ncol=nrep)
IC <- lapply(1:6, matrix, data= 0, nrow=21, ncol=nrep)

sigma<- lapply(1:5, matrix, data= 0, nrow=21, ncol=nrep)

b_bt <- lapply(1:6, matrix, data= 0, nrow=21, ncol=nrep)
bS_bt <- lapply(1:6, matrix, data= 0, nrow=21, ncol=nrep)
bF_bt <- lapply(1:6, matrix, data= 0, nrow=21, ncol=nrep)
bT_bt <- lapply(1:6, matrix, data= 0, nrow=21, ncol=nrep)
PrSur_bt <- lapply(1:6, matrix, data= 0, nrow=21, ncol=nrep)

for (i in 1:5) {
  sigma[[i]] <- as.xts(sigma[[i]],order.by=dates7)
}


for (i in 1:6)
{ r_bt[[i]] <- as.xts(r_bt[[i]],order.by=dates7) 
r_10Y_bt[[i]] <- as.xts(r_10Y_bt[[i]],order.by=dates7)
g_bt[[i]] <- as.xts(g_bt[[i]],order.by=dates7)
GDP[[i]] <- as.xts(GDP[[i]],order.by=dates7)
W[[i]] <- as.xts(W[[i]],order.by=dates7)
IC[[i]] <- as.xts(IC[[i]],order.by=dates7)

b_bt[[i]] <- as.xts(b_bt[[i]],order.by=dates7)
bS_bt[[i]] <- as.xts(bS_bt[[i]],order.by=dates7)
bF_bt[[i]] <- as.xts(bF_bt[[i]],order.by=dates7)
bT_bt[[i]] <- as.xts(bT_bt[[i]],order.by=dates7)
PrSur_bt[[i]] <- as.xts(PrSur_bt[[i]],order.by=dates7)}

names(r_bt) <- name
names(r_10Y_bt) <- name
names(g_bt) <- name
names(GDP) <- name
names(W) <- name
names(IC) <- name

names(sigma) <- Names

names(b_bt) <- name
names(bS_bt) <- name
names(bF_bt) <- name
names(bT_bt) <- name
names(PrSur_bt) <- name

for (Name in MSList[MSList$Simulation=="Y",4])
{ r_bt[[Name]][1,] <- MyData_subset[k,paste(Name, "CM", sep="_")]
r_10Y_bt[[Name]][1,] <- MyData_subset[k,paste(Name, "10Y", sep="_")]
g_bt[[Name]][1,] <- MyData_subset[k,paste(Name, "Growth", sep="_")]
GDP[[Name]][1,] <- MyData_subset[k,paste(Name, "GDP", sep="_")]
W[[Name]][1,] <- MyData_subset[k,paste(Name, "W", sep="_")]
IC[[Name]][1,] <- MyData[k1,paste(Name, "IC", sep="_")][[1]]

b_bt[[Name]][1,] <- SimDebt1[k2,paste(Name, "B", sep="_")][[1]]/MyData[k1,paste(Name, "GDP", sep="_")][[1]]
bS_bt[[Name]][1,] <- SimDebt1[k2,paste(Name, "bS", sep="_")][[1]] 
bF_bt[[Name]][1,] <- SimDebt1[k2,paste(Name, "bF", sep="_")][[1]]
bT_bt[[Name]][1,] <- b_bt[[Name]][1,][[1]] - 10*beta*(bF_bt[[Name]][1,][[1]] - b*(bF_bt[[Name]][1,][[1]]/(bF_bt[[Name]][1,][[1]]+bS_bt[[Name]][1,][[1]]))) +
  -10*gamma*(bS_bt[[Name]][1,][[1]]- b*(1- (bF_bt[[Name]][1,][[1]]/(bF_bt[[Name]][1,][[1]]+bS_bt[[Name]][1,][[1]]))))
PrSur_bt[[Name]][1,] <- MyData[k1,paste(Name, "PrimarySurplus", sep="_")][[1]]/MyData[k1,paste(Name, "GDP", sep="_")][[1]]
}

AGG_GDP<- as.xts(array(NA, c(21, nrep)), order.by=dates7)
AGG_GDP[1,] <- MyData_subset[k,"AGG_GDP"][[1]]
AGG_Gr<- as.xts(array(NA, c(21, nrep)), order.by=dates7)
AGG_Gr[1,] <- MyData_subset[k,"AGG_Gr"][[1]]

IR1_bt <- as.xts(array(NA, c(21, nrep)), order.by=dates7)
IR1_bt[1,] <- MyData_subset[k,"IR1"]
IR10_bt <- as.xts(array(NA, c(21, nrep)), order.by=dates7)
IR10_bt[1,] <- MyData_subset[k,"IR10"]

phi <- as.xts(array(0, c(21, nrep)), order.by=dates7)

for (i in 1:nrep){
  
  for (j in 2:21) {
    
    for (Name in MSList[MSList$Simulation=="Y",4])
    { g_bt[[Name]][j,i][[1]] <- Coeff_G[1, paste(Name)] + Coeff_G[2, paste(Name)]*IR1_bt[j-1,i][[1]] +
      + Coeff_G[3, paste(Name)]*(r_10Y_bt[[Name]][j-1,i][[1]]- r_10Y_bt[["DE"]][j-1,i][[1]]) +
      + Coeff_G[4, paste(Name)]*PrSur_bt[[Name]][j-1,i][[1]] + res1_bt[[Name]][j,i][[1]] 
    
    GDP[[Name]][j,i][[1]] <- (1+g_bt[[Name]][j,i][[1]])*GDP[[Name]][j-1,i][[1]] }
    
    
    AGG_GDP[j,i][[1]]<- GDP[["DE"]][j,i][[1]]+GDP[["ES"]][j,i][[1]]+GDP[["FR"]][j,i][[1]]+
      GDP[["GR"]][j,i][[1]]+GDP[["IT"]][j,i][[1]]+GDP[["PT"]][j,i][[1]]
    
    W[["DE"]][j,i][[1]]<- GDP[["DE"]][j,i][[1]]/AGG_GDP[j,i][[1]]
    W[["ES"]][j,i][[1]]<- GDP[["ES"]][j,i][[1]]/AGG_GDP[j,i][[1]]
    W[["FR"]][j,i][[1]]<- GDP[["FR"]][j,i][[1]]/AGG_GDP[j,i][[1]]
    W[["GR"]][j,i][[1]]<- GDP[["GR"]][j,i][[1]]/AGG_GDP[j,i][[1]]
    W[["IT"]][j,i][[1]]<- GDP[["IT"]][j,i][[1]]/AGG_GDP[j,i][[1]]
    W[["PT"]][j,i][[1]]<- GDP[["PT"]][j,i][[1]]/AGG_GDP[j,i][[1]]
    
    AGG_Gr[j,i][[1]] <- W[["DE"]][j-1,i][[1]]*g_bt[["DE"]][j,i][[1]] + W[["ES"]][j-1,i][[1]]*g_bt[["ES"]][j,i][[1]] +
      + W[["FR"]][j-1,i][[1]]*g_bt[["FR"]][j,i][[1]] + W[["GR"]][j-1,i][[1]]*g_bt[["GR"]][j,i][[1]] + 
      + W[["IT"]][j-1,i][[1]]*g_bt[["IT"]][j,i][[1]] + W[["PT"]][j-1,i][[1]]*g_bt[["PT"]][j,i][[1]]
    
    IR1_bt[j,i][[1]] <- b0 + b1*AGG_Gr[j,i][[1]] + b2*IR1_bt[j-1,i][[1]] + res2_bt[j,i]
    
    IR10_bt[j,i][[1]] <- c0 + c1*IR10_bt[j-1,i][[1]] + c2*IR1_bt[j,i][[1]] + res3_bt[j,i]
    
    
    r_10Y_bt[["DE"]][j,i][[1]] <- Coeff_DE[1,]+ Coeff_DE[1,]*r_10Y_bt[["DE"]][j-1,i][[1]] + 
      + Coeff_DE[3,]*IR1_bt[j,i][[1]] + res6_bt[j,i][[1]]
    
    r_bt[["DE"]][j,i][[1]] <- Coeff_CM[1,"DE"]*r_bt[["DE"]][j-1,i][[1]] + Coeff_CM[2, "DE"]*r_10Y_bt[["DE"]][j,i][[1]] +
      + Coeff_CM[3,"DE"]*IR1_bt[j,i][[1]] + res4_bt[["DE"]][j,i][[1]]
    
    for (Name in Names)
    { sigma[[Name]][j,i][[1]] = 0.004*b_bt[[Name]][j-1,i][[1]]
      r_10Y_bt[[Name]][j,i][[1]] <- r_10Y_bt[["DE"]][j,i][[1]] + Coeff_spread[1, paste(Name)] +
      + Coeff_spread[2, paste(Name)]*b_bt[[Name]][j-1,i][[1]] + res5_EDA_bt[[Name]][j,i][[1]]*sigma[[Name]][j,i][[1]]
    
    r_bt[[Name]][j,i][[1]] <- Coeff_CM[1,paste(Name)]*r_bt[[Name]][j-1,i][[1]] + Coeff_CM[2, paste(Name)]*r_10Y_bt[[Name]][j,i][[1]] +
      + Coeff_CM[3,paste(Name)]*IR1_bt[j,i][[1]] +res4_bt[[Name]][j,i][[1]] 
    }
    
    for (Name in MSList[MSList$Simulation=="Y",4])
    {
      RF <- IR10_bt[j,i][[1]]
      alpha <- 1/(1+RF)
      B <- D_TTC/(1+RF)
      Bfirst <- unitVector-diag(B)
      Bfirst <- B/Bfirst 
      GrossPerpetuity <- alpha/(1-alpha)*unitVector
      GrossPerpetuity <- GrossPerpetuity-S_TTC%*%Bfirst%*%S_TTC_Inv%*%v  ### eq. 8 
      
      D_TTCInv <- solve(D_TTC)
      temp <- (UnitMatrix-D_TTCInv)%*%Bfirst
      temp <- rr*S_TTC%*%temp%*%S_TTC_Inv%*%v ### eq. 9
      temp <- (unitVector- temp)
      NetPerpetuity <- GrossPerpetuity/temp ### eq. 10
      cf <- 1/NetPerpetuity ### eq. 13 annual fundamental pricing
      
      MSCurrentRating <- Rating[, Name][[1]]
      PriceRecord <- Ratings[Ratings$Grade==MSCurrentRating,2]
      IC[[Name]][j,i][[1]] <- cf[PriceRecord]
    }
    
    for (Name in MSList[MSList$Simulation=="Y",4])
    {
      PrSur_bt[[Name]][j,i][[1]] = b_bt[[Name]][j-1,i][[1]]*((r_bt[[Name]][j,i][[1]]+
                                                                - g_bt[[Name]][j,i][[1]])/(1+g_bt[[Name]][j,i][[1]])) +
        + delta*(b_bt[[Name]][j-1, i][[1]] -bT_bt[[Name]][j-1, i][[1]])
      
      b_bt[[Name]][j, i][[1]] = b_bt[[Name]][j-1, i][[1]]* (1+ ((r_bt[[Name]][j,i][[1]] +
                                                                   - g_bt[[Name]][j,i][[1]])/(1+g_bt[[Name]][j,i][[1]]))) +
        - (1-phi[j, i][[1]])*PrSur_bt[[Name]][j, i][[1]] 
      
      bS_bt[[Name]][j, i][[1]] =  bS_bt[[Name]][j-1, i][[1]]*((1+r_bt[[Name]][j,i][[1]])/(1+g_bt[[Name]][j,i][[1]])) +
        - phi[j, i][[1]]*(PrSur_bt[[Name]][j, i][[1]])
      
      bF_bt[[Name]][j, i][[1]] = b_bt[[Name]][j, i][[1]] - bS_bt[[Name]][j, i][[1]]
      
      bT_bt[[Name]][j, i][[1]] <- b_bt[[Name]][j, i][[1]] - 10*beta*(bF_bt[[Name]][j, i][[1]]+
                                                                       - b*(bF_bt[[Name]][j, i][[1]]/(bF_bt[[Name]][j, i][[1]]+bS_bt[[Name]][j, i][[1]]))) +
        -10*gamma*(bS_bt[[Name]][j, i][[1]]- b*(1- (bF_bt[[Name]][j, i][[1]]/(bF_bt[[Name]][j, i][[1]]+bS_bt[[Name]][j, i][[1]]))))
      
    }
  }}

for (Name in MSList[MSList$Simulation=="Y",4])
{ temp <- list(r_bt[[Name]], r_10Y_bt[[Name]], g_bt[[Name]],GDP[[Name]],W[[Name]], IC[[Name]],
               b_bt[[Name]], bS_bt[[Name]], bF_bt[[Name]],bT_bt[[Name]],PrSur_bt[[Name]])
names(temp) <- c("r_bt", "r_10Y_bt", "g_bt","GDP","W", "IC",
                 "b_bt", "bS_bt", "bF_bt","bT_bt","PrSur_bt")
temp <- assign( paste("List_S0",Name, sep="_") , temp )}



rm(temp,r_bt, r_10Y_bt, g_bt,GDP,W, IC,b_bt, bS_bt,bF_bt,bT_bt,PrSur_bt)

###----------------------
### EDA SCENARIO - SMALL EDA 
###---------------------- 

r_bt <- lapply(1:6, matrix, data= 0, nrow=21, ncol=nrep)
r_10Y_bt <- lapply(1:6, matrix, data= 0, nrow=21, ncol=nrep)
g_bt <- lapply(1:6, matrix, data= 0, nrow=21, ncol=nrep)
GDP <- lapply(1:6, matrix, data= 0, nrow=21, ncol=nrep)
W <- lapply(1:6, matrix, data= 0, nrow=21, ncol=nrep)
IC <- lapply(1:6, matrix, data= 0, nrow=21, ncol=nrep)

sigma<- lapply(1:5, matrix, data= 0, nrow=21, ncol=nrep)

b_bt <- lapply(1:6, matrix, data= 0, nrow=21, ncol=nrep)
bS_bt <- lapply(1:6, matrix, data= 0, nrow=21, ncol=nrep)
bF_bt <- lapply(1:6, matrix, data= 0, nrow=21, ncol=nrep)
bT_bt <- lapply(1:6, matrix, data= 0, nrow=21, ncol=nrep)
PrSur_bt <- lapply(1:6, matrix, data= 0, nrow=21, ncol=nrep)
d_bt <- lapply(1:6, matrix, data= 0, nrow=21, ncol=nrep)
dT_bt <- lapply(1:6, matrix, data= 0, nrow=21, ncol=nrep)


r_ECB_bt <- lapply(1:6, matrix, data= 0, nrow=21, ncol=nrep)
P_EDA_bt <- lapply(1:6, matrix, data= 0, nrow=21, ncol=nrep)
NL_EDA_bt <- lapply(1:6, matrix, data= 0, nrow=21, ncol=nrep)
L_EDA_bt <- lapply(1:6, matrix, data= 0, nrow=21, ncol=nrep)
b_EDA_bt <- lapply(1:6, matrix, data= 0, nrow=21, ncol=nrep)
EL_EDA_bt <- lapply(1:6, matrix, data= 0, nrow=21, ncol=nrep)
RY_EDA_bt <- lapply(1:6, matrix, data= 0, nrow=21, ncol=nrep)
EN_EDA_bt <- lapply(1:6, matrix, data= 0, nrow=21, ncol=nrep)
SC_EDA_bt <- lapply(1:6, matrix, data= 0, nrow=21, ncol=nrep)
LT_EDA_bt <- lapply(1:6, matrix, data= 0, nrow=21, ncol=nrep)

for (i in 1:5) {
  sigma[[i]] <- as.xts(sigma[[i]],order.by=dates7)
}

for (i in 1:6)
{ r_bt[[i]] <- as.xts(r_bt[[i]],order.by=dates7) 
r_10Y_bt[[i]] <- as.xts(r_10Y_bt[[i]],order.by=dates7)
g_bt[[i]] <- as.xts(g_bt[[i]],order.by=dates7)
GDP[[i]] <- as.xts(GDP[[i]],order.by=dates7)
W[[i]] <- as.xts(W[[i]],order.by=dates7)
IC[[i]] <- as.xts(IC[[i]],order.by=dates7)

b_bt[[i]] <- as.xts(b_bt[[i]],order.by=dates7)
bS_bt[[i]] <- as.xts(bS_bt[[i]],order.by=dates7)
bF_bt[[i]] <- as.xts(bF_bt[[i]],order.by=dates7)
bT_bt[[i]] <- as.xts(bT_bt[[i]],order.by=dates7)
PrSur_bt[[i]] <- as.xts(PrSur_bt[[i]],order.by=dates7)
d_bt[[i]] <- as.xts(d_bt[[i]],order.by=dates7)
dT_bt[[i]] <- as.xts(dT_bt[[i]],order.by=dates7)


P_EDA_bt[[i]] <- as.xts(P_EDA_bt[[i]],order.by=dates7)
NL_EDA_bt[[i]] <- as.xts(NL_EDA_bt[[i]],order.by=dates7)
L_EDA_bt[[i]] <- as.xts(L_EDA_bt[[i]],order.by=dates7)
b_EDA_bt[[i]] <- as.xts(b_EDA_bt[[i]],order.by=dates7)
EL_EDA_bt[[i]] <- as.xts(EL_EDA_bt[[i]],order.by=dates7)
RY_EDA_bt[[i]] <- as.xts(RY_EDA_bt[[i]],order.by=dates7)
EN_EDA_bt[[i]] <- as.xts(EN_EDA_bt[[i]],order.by=dates7)
SC_EDA_bt[[i]] <- as.xts(SC_EDA_bt[[i]],order.by=dates7)
LT_EDA_bt[[i]] <- as.xts(LT_EDA_bt[[i]],order.by=dates7)}

names(r_bt) <- name
names(r_10Y_bt) <- name
names(g_bt) <- name
names(GDP) <- name
names(W) <- name
names(IC) <- name
names(sigma) <- Names

names(b_bt) <- name
names(bS_bt) <- name
names(bF_bt) <- name
names(bT_bt) <- name
names(PrSur_bt) <- name
names(d_bt) <- name
names(dT_bt) <- name

names(r_ECB_bt) <- name
names(P_EDA_bt) <- name
names(NL_EDA_bt) <- name
names(L_EDA_bt) <- name
names(b_EDA_bt) <- name
names(EL_EDA_bt) <- name
names(RY_EDA_bt) <- name
names(EN_EDA_bt) <- name
names(SC_EDA_bt) <- name
names(LT_EDA_bt) <- name


for (Name in MSList[MSList$Simulation=="Y",4])
{ r_bt[[Name]][1,] <- MyData_subset[k,paste(Name, "CM", sep="_")]
r_10Y_bt[[Name]][1,] <- MyData_subset[k,paste(Name, "10Y", sep="_")]
g_bt[[Name]][1,] <- MyData_subset[k,paste(Name, "Growth", sep="_")]
GDP[[Name]][1,] <- MyData_subset[k,paste(Name, "GDP", sep="_")]
W[[Name]][1,] <- MyData_subset[k,paste(Name, "W", sep="_")]
IC[[Name]][1,] <- MyData[k1,paste(Name, "IC", sep="_")][[1]]

b_bt[[Name]][1,] <- SimDebt1[k2,paste(Name, "B", sep="_")][[1]]/MyData[k1,paste(Name, "GDP", sep="_")][[1]]
bS_bt[[Name]][1,] <- SimDebt1[k2,paste(Name, "bS", sep="_")][[1]] 
bF_bt[[Name]][1,] <- SimDebt1[k2,paste(Name, "bF", sep="_")][[1]]
bT_bt[[Name]][1,] <- 0
PrSur_bt[[Name]][1,] <- MyData[k1,paste(Name, "PrimarySurplus", sep="_")][[1]]/MyData[k1,paste(Name, "GDP", sep="_")][[1]]

r_ECB_bt[[Name]][1,]<- g_bt[[Name]][1,]
P_EDA_bt[[Name]][1,] <- 0
NL_EDA_bt[[Name]][1,] <- 0
L_EDA_bt[[Name]][1,] <- 0
b_EDA_bt[[Name]][1,] <- 0
EL_EDA_bt[[Name]][1,] <- 0
RY_EDA_bt[[Name]][1,] <- 0
EN_EDA_bt[[Name]][1,] <- 0
SC_EDA_bt[[Name]][1,] <- 0
LT_EDA_bt[[Name]][1,] <- 0

d_bt[[Name]][1,] <- b_bt[[Name]][1,] + L_EDA_bt[[Name]][1,]

dT_bt[[Name]][1, ] <- b_bt[[Name]][1,] - 10*beta*(bF_bt[[Name]][1,]+
                                                    - b*(bF_bt[[Name]][1,]/(bF_bt[[Name]][1,]+bS_bt[[Name]][1,]))) +
  -10*gamma*(bS_bt[[Name]][1,]- b*(1- (bF_bt[[Name]][1,]/(bF_bt[[Name]][1,]+bS_bt[[Name]][1,]))))
}


AGG_GDP_EDA<- as.xts(array(NA, c(21, nrep)), order.by=dates7)
AGG_GDP_EDA[1,] <- MyData_subset[k,"AGG_GDP"][[1]]
AGG_Gr_EDA<- as.xts(array(NA, c(21, nrep)), order.by=dates7)
AGG_Gr_EDA[1,] <- MyData_subset[k,"AGG_Gr"][[1]]

IR1_bt_EDA <- as.xts(array(NA, c(21, nrep)), order.by=dates7)
IR1_bt_EDA[1,] <- MyData_subset[k,"IR1"]
IR10_bt_EDA <- as.xts(array(NA, c(21, nrep)), order.by=dates7)
IR10_bt_EDA[1,] <- MyData_subset[k,"IR10"]

phi <- as.xts(array(0, c(21, nrep)), order.by=dates7)

for (i in 1:nrep){
  
  for (j in 2) {
    
    for (Name in MSList[MSList$Simulation=="Y",4])
    { g_bt[[Name]][j,i][[1]] <- Coeff_G[1, paste(Name)] + Coeff_G[2, paste(Name)]*IR1_bt_EDA[j-1,i][[1]] +
      + Coeff_G[3, paste(Name)]*(r_10Y_bt[[Name]][j-1,i][[1]]- r_10Y_bt[["DE"]][j-1,i][[1]]) +
      + Coeff_G[4, paste(Name)]*PrSur_bt[[Name]][j-1,i][[1]] + res1_bt[[Name]][j,i][[1]] 
    
    GDP[[Name]][j,i][[1]] <- (1+g_bt[[Name]][j,i][[1]])*GDP[[Name]][j-1,i][[1]] }
    
    
    AGG_GDP_EDA[j,i][[1]]<- GDP[["DE"]][j,i][[1]]+GDP[["ES"]][j,i][[1]]+GDP[["FR"]][j,i][[1]]+
      GDP[["GR"]][j,i][[1]]+GDP[["IT"]][j,i][[1]]+GDP[["PT"]][j,i][[1]]
    
    W[["DE"]][j,i][[1]]<- GDP[["DE"]][j,i][[1]]/AGG_GDP_EDA[j,i][[1]]
    W[["ES"]][j,i][[1]]<- GDP[["ES"]][j,i][[1]]/AGG_GDP_EDA[j,i][[1]]
    W[["FR"]][j,i][[1]]<- GDP[["FR"]][j,i][[1]]/AGG_GDP_EDA[j,i][[1]]
    W[["GR"]][j,i][[1]]<- GDP[["GR"]][j,i][[1]]/AGG_GDP_EDA[j,i][[1]]
    W[["IT"]][j,i][[1]]<- GDP[["IT"]][j,i][[1]]/AGG_GDP_EDA[j,i][[1]]
    W[["PT"]][j,i][[1]]<- GDP[["PT"]][j,i][[1]]/AGG_GDP_EDA[j,i][[1]]
    
    AGG_Gr_EDA[j,i][[1]] <- W[["DE"]][j-1,i][[1]]*g_bt[["DE"]][j,i][[1]] + W[["ES"]][j-1,i][[1]]*g_bt[["ES"]][j,i][[1]] +
      + W[["FR"]][j-1,i][[1]]*g_bt[["FR"]][j,i][[1]] + W[["GR"]][j-1,i][[1]]*g_bt[["GR"]][j,i][[1]] + 
      + W[["IT"]][j-1,i][[1]]*g_bt[["IT"]][j,i][[1]] + W[["PT"]][j-1,i][[1]]*g_bt[["PT"]][j,i][[1]]
    
    IR1_bt_EDA[j,i][[1]] <- b0 + b1*AGG_Gr_EDA[j,i][[1]] + b2*IR1_bt_EDA[j-1,i][[1]] + res2_bt[j,i]
    
    IR10_bt_EDA[j,i][[1]] <- c0 + c1*IR10_bt_EDA[j-1,i][[1]] + c2*IR1_bt_EDA[j,i][[1]] + res3_bt[j,i]
    
    r_ECB_bt[[Name]][j,i][[1]] <- g_bt[[Name]][j,i][[1]]
    
    r_10Y_bt[["DE"]][j,i][[1]] <- Coeff_DE[1,]+ Coeff_DE[1,]*r_10Y_bt[["DE"]][j-1,i][[1]] + 
      + Coeff_DE[3,]*IR1_bt[j,i][[1]] + res6_bt[j,i][[1]]
    
    r_bt[["DE"]][j,i][[1]] <- Coeff_CM[1,"DE"]*r_bt[["DE"]][j-1,i][[1]] + Coeff_CM[2, "DE"]*r_10Y_bt[["DE"]][j,i][[1]] +
      + Coeff_CM[3,"DE"]*IR1_bt[j,i][[1]] + res4_bt[["DE"]][j,i][[1]]
    
    
    for (Name in Names)
    { sigma[[Name]][j,i][[1]] = 0.004*b_bt[[Name]][j-1,i][[1]]
      
      r_10Y_bt[[Name]][j,i][[1]] <- r_10Y_bt[["DE"]][j,i][[1]] + Coeff_spread[1, paste(Name)] +
      + Coeff_spread[2, paste(Name)]*b_bt[[Name]][j-1,i][[1]] + res5_EDA_bt[[Name]][j,i][[1]]*sigma[[Name]][j,i][[1]]
 
    r_bt[[Name]][j,i][[1]] <- Coeff_CM[1,paste(Name)]*r_bt[[Name]][j-1,i][[1]] + Coeff_CM[2, paste(Name)]*r_10Y_bt[[Name]][j,i][[1]] +
      + Coeff_CM[3,paste(Name)]*IR1_bt[j,i][[1]] +res4_bt[[Name]][j,i][[1]]}
    
    for (Name in MSList[MSList$Simulation=="Y",4])
    {
      RF <- IR10_bt_EDA[j,i][[1]]
      alpha <- 1/(1+RF)
      B <- D_TTC/(1+RF)
      Bfirst <- unitVector-diag(B)
      Bfirst <- B/Bfirst 
      GrossPerpetuity <- alpha/(1-alpha)*unitVector
      GrossPerpetuity <- GrossPerpetuity-S_TTC%*%Bfirst%*%S_TTC_Inv%*%v  ### eq. 8 
      
      D_TTCInv <- solve(D_TTC)
      temp <- (UnitMatrix-D_TTCInv)%*%Bfirst
      temp <- rr*S_TTC%*%temp%*%S_TTC_Inv%*%v ### eq. 9
      temp <- (unitVector- temp)
      NetPerpetuity <- GrossPerpetuity/temp ### eq. 10
      cf <- 1/NetPerpetuity ### eq. 13 annual fundamental pricing
      
      MSCurrentRating <- Rating[, Name][[1]]
      PriceRecord <- Ratings[Ratings$Grade==MSCurrentRating,2]
      IC[[Name]][j,i][[1]] <- cf[PriceRecord]
    }
    
    for (Name in MSList[MSList$Simulation=="Y",4])
    {
      K <- 704799*MES[MES$Name==Name,2][[1]]/100
      
      P_EDA_bt[[Name]][j,i][[1]]<-0
      
      PrSur_bt[[Name]][j,i][[1]] = b_bt[[Name]][j-1,i][[1]]*((r_bt[[Name]][j,i][[1]]- g_bt[[Name]][j,i][[1]])/(1+g_bt[[Name]][j,i][[1]])) +
        + delta*(d_bt[[Name]][j-1,i][[1]] -dT_bt[[Name]][j-1,i][[1]])
      
      
      NL_EDA_bt[[Name]][j,i][[1]] <- 1/5*(1/(1+g_bt[[Name]][j,i][[1]]))*bS_bt[[Name]][j-1,i][[1]] +
        + r_bt[[Name]][j,i][[1]]/(1+g_bt[[Name]][j,i][[1]])*bS_bt[[Name]][j-1,i][[1]] +
        - phi[j, i][[1]]*(PrSur_bt[[Name]][j,i][[1]])
      
      L_EDA_bt[[Name]][j,i][[1]] <- NL_EDA_bt[[Name]][j,i][[1]]
      b_EDA_bt[[Name]][j,i][[1]] <- NL_EDA_bt[[Name]][j,i][[1]]
      EL_EDA_bt[[Name]][j,i][[1]] <- 0
      RY_EDA_bt[[Name]][j,i][[1]] <- 0
      EN_EDA_bt[[Name]][j,i][[1]] <- K/GDP[[Name]][j,i][[1]] 
      SC_EDA_bt[[Name]][j,i][[1]] <- L_EDA_bt[[Name]][j,i][[1]]+RY_EDA_bt[[Name]][j,i][[1]]+EN_EDA_bt[[Name]][j,i][[1]]+
        - b_EDA_bt[[Name]][j,i][[1]] -  EL_EDA_bt[[Name]][j,i][[1]]
      b_bt[[Name]][j,i][[1]] = b_bt[[Name]][j-1,i][[1]]* (1+ ((r_bt[[Name]][j,i][[1]] +
                                                                 - g_bt[[Name]][j,i][[1]])/(1+g_bt[[Name]][j,i][[1]]))) +
        - (1-phi[j, i][[1]])*PrSur_bt[[Name]][j,i][[1]] +
        - (1/5*(1/(1+g_bt[[Name]][j,i][[1]]))*bS_bt[[Name]][j-1,i][[1]] +
             + r_bt[[Name]][j,i][[1]]/(1+g_bt[[Name]][j,i][[1]])*bS_bt[[Name]][j-1,i][[1]])
      
      bS_bt[[Name]][j,i][[1]] = (4/5)*bS_bt[[Name]][j-1,i][[1]]* (1/(1+g_bt[[Name]][j,i][[1]])) 
      
      bF_bt[[Name]][j,i][[1]] = b_bt[[Name]][j,i][[1]] - bS_bt[[Name]][j,i][[1]] 
      
      LT_EDA_bt[[Name]][j,i][[1]] = 0.6*L_EDA_bt[[Name]][j,i][[1]]/(bF_bt[[Name]][j,i][[1]]+bS_bt[[Name]][j,i][[1]]+L_EDA_bt[[Name]][j,i][[1]])
      
      d_bt[[Name]][j,i][[1]] = b_bt[[Name]][j,i][[1]] + L_EDA_bt[[Name]][j,i][[1]]
      dT_bt[[Name]][j,i][[1]] = d_bt[[Name]][j,i][[1]] +
        - 10*beta*(bF_bt[[Name]][j,i][[1]]+
                     - b*(bF_bt[[Name]][j,i][[1]]/(bF_bt[[Name]][j,i][[1]]+bS_bt[[Name]][j,i][[1]]+L_EDA_bt[[Name]][j,i][[1]]))) +
        - 10*gamma*(bS_bt[[Name]][j,i][[1]]- b*(1- (bF_bt[[Name]][j,i][[1]]/(bF_bt[[Name]][j,i][[1]]+bS_bt[[Name]][j,i][[1]]+L_EDA_bt[[Name]][j,i][[1]]))))
      - 10*gamma*(L_EDA_bt[[Name]][j,i][[1]] - LT_EDA_bt[[Name]][j,i][[1]])
    }}
  
  m= 4
  for (j in 3:6) {
    
    for (Name in MSList[MSList$Simulation=="Y",4])
    { g_bt[[Name]][j,i][[1]] <- Coeff_G[1, paste(Name)] + Coeff_G[2, paste(Name)]*IR1_bt_EDA[j-1,i][[1]] +
      + Coeff_G[3, paste(Name)]*(r_10Y_bt[[Name]][j-1,i][[1]]- r_10Y_bt[["DE"]][j-1,i][[1]]) +
      + Coeff_G[4, paste(Name)]*PrSur_bt[[Name]][j-1,i][[1]] + res1_bt[[Name]][j,i][[1]]  
    
    GDP[[Name]][j,i][[1]] <- (1+g_bt[[Name]][j,i][[1]])*GDP[[Name]][j-1,i][[1]] }
    
    
    AGG_GDP_EDA[j,i][[1]]<- GDP[["DE"]][j,i][[1]]+GDP[["ES"]][j,i][[1]]+GDP[["FR"]][j,i][[1]]+
      GDP[["GR"]][j,i][[1]]+GDP[["IT"]][j,i][[1]]+GDP[["PT"]][j,i][[1]]
    
    W[["DE"]][j,i][[1]]<- GDP[["DE"]][j,i][[1]]/AGG_GDP_EDA[j,i][[1]]
    W[["ES"]][j,i][[1]]<- GDP[["ES"]][j,i][[1]]/AGG_GDP_EDA[j,i][[1]]
    W[["FR"]][j,i][[1]]<- GDP[["FR"]][j,i][[1]]/AGG_GDP_EDA[j,i][[1]]
    W[["GR"]][j,i][[1]]<- GDP[["GR"]][j,i][[1]]/AGG_GDP_EDA[j,i][[1]]
    W[["IT"]][j,i][[1]]<- GDP[["IT"]][j,i][[1]]/AGG_GDP_EDA[j,i][[1]]
    W[["PT"]][j,i][[1]]<- GDP[["PT"]][j,i][[1]]/AGG_GDP_EDA[j,i][[1]]
    
    AGG_Gr_EDA[j,i][[1]] <- W[["DE"]][j-1,i][[1]]*g_bt[["DE"]][j,i][[1]] + W[["ES"]][j-1,i][[1]]*g_bt[["ES"]][j,i][[1]] +
      + W[["FR"]][j-1,i][[1]]*g_bt[["FR"]][j,i][[1]] + W[["GR"]][j-1,i][[1]]*g_bt[["GR"]][j,i][[1]] + 
      + W[["IT"]][j-1,i][[1]]*g_bt[["IT"]][j,i][[1]] + W[["PT"]][j-1,i][[1]]*g_bt[["PT"]][j,i][[1]]
    
    IR1_bt_EDA[j,i][[1]] <- b0 + b1*AGG_Gr_EDA[j,i][[1]] + b2*IR1_bt_EDA[j-1,i][[1]] + res2_bt[j,i]
    
    IR10_bt_EDA[j,i][[1]] <- c0 + c1*IR10_bt_EDA[j-1,i][[1]] + c2*IR1_bt_EDA[j,i][[1]] + res3_bt[j,i]
    
    r_ECB_bt[[Name]][j,i][[1]] <- g_bt[[Name]][j,i][[1]]
    
    r_10Y_bt[["DE"]][j,i][[1]] <- Coeff_DE[1,]+ Coeff_DE[1,]*r_10Y_bt[["DE"]][j-1,i][[1]] + 
      + Coeff_DE[3,]*IR1_bt[j,i][[1]] + res6_bt[j,i][[1]]
    
    r_bt[["DE"]][j,i][[1]] <- Coeff_CM[1,"DE"]*r_bt[["DE"]][j-1,i][[1]] + Coeff_CM[2, "DE"]*r_10Y_bt[["DE"]][j,i][[1]] +
      + Coeff_CM[3,"DE"]*IR1_bt[j,i][[1]] + res4_bt[["DE"]][j,i][[1]]
    
    for (Name in Names)
    { sigma[[Name]][j,i][[1]] = 0.004*b_bt[[Name]][j-1,i][[1]]
    
      r_10Y_bt[[Name]][j,i][[1]] <- r_10Y_bt[["DE"]][j,i][[1]] + Coeff_spread[1, paste(Name)] +
      + Coeff_spread[2, paste(Name)]*b_bt[[Name]][j-1,i][[1]] + res5_EDA_bt[[Name]][j,i][[1]]*sigma[[Name]][j,i][[1]] 
    
    r_bt[[Name]][j,i][[1]] <- Coeff_CM[1,paste(Name)]*r_bt[[Name]][j-1,i][[1]] + Coeff_CM[2, paste(Name)]*r_10Y_bt[[Name]][j,i][[1]] +
      + Coeff_CM[3,paste(Name)]*IR1_bt[j,i][[1]] +res4_bt[[Name]][j,i][[1]] 
    }
    
    for (Name in MSList[MSList$Simulation=="Y",4])
    {
      RF <- IR10_bt_EDA[j,i][[1]]
      alpha <- 1/(1+RF)
      B <- D_TTC/(1+RF)
      Bfirst <- unitVector-diag(B)
      Bfirst <- B/Bfirst 
      GrossPerpetuity <- alpha/(1-alpha)*unitVector
      GrossPerpetuity <- GrossPerpetuity-S_TTC%*%Bfirst%*%S_TTC_Inv%*%v  ### eq. 8 
      
      D_TTCInv <- solve(D_TTC)
      temp <- (UnitMatrix-D_TTCInv)%*%Bfirst
      temp <- rr*S_TTC%*%temp%*%S_TTC_Inv%*%v ### eq. 9
      temp <- (unitVector- temp)
      NetPerpetuity <- GrossPerpetuity/temp ### eq. 10
      cf <- 1/NetPerpetuity ### eq. 13 annual fundamental pricing
      
      MSCurrentRating <- Rating[, Name][[1]]
      PriceRecord <- Ratings[Ratings$Grade==MSCurrentRating,2]
      IC[[Name]][j,i][[1]] <- cf[PriceRecord]
    }
    
    
    for (Name in MSList[MSList$Simulation=="Y",4])
    {   
      P_EDA_bt[[Name]][j,i][[1]]<- IC[[Name]][j,i][[1]]*(b_EDA_bt[[Name]][j-1,i][[1]] - RY_EDA_bt[[Name]][j-1,i][[1]])/(1+g_bt[[Name]][j,i][[1]])
      
      
      PrSur_bt[[Name]][j,i][[1]] = P_EDA_bt[[Name]][j,i][[1]] +
        + b_bt[[Name]][j-1,i][[1]]*((r_bt[[Name]][j,i][[1]]- g_bt[[Name]][j,i][[1]])/(1+g_bt[[Name]][j,i][[1]])) +
        - g_bt[[Name]][j,i][[1]]/(1+g_bt[[Name]][j,i][[1]])*L_EDA_bt[[Name]][j-1,i][[1]] +
        + delta*(d_bt[[Name]][j-1,i][[1]] -dT_bt[[Name]][j-1,i][[1]])
      
      
      NL_EDA_bt[[Name]][j,i][[1]] <- 1/m*(1/(1+g_bt[[Name]][j,i][[1]]))*bS_bt[[Name]][j-1,i][[1]] +
        + r_bt[[Name]][j,i][[1]]/(1+g_bt[[Name]][j,i][[1]])*bS_bt[[Name]][j-1,i][[1]] +
        - phi[j,i][[1]]*(PrSur_bt[[Name]][j,i][[1]])+
        + P_EDA_bt[[Name]][j,i][[1]]
      
      L_EDA_bt[[Name]][j,i][[1]] <- (1/(1+g_bt[[Name]][j,i][[1]]))* L_EDA_bt[[Name]][j-1,i][[1]] + NL_EDA_bt[[Name]][j,i][[1]]
      b_EDA_bt[[Name]][j,i][[1]] <- b_EDA_bt[[Name]][j-1,i][[1]]*((1+IR10_bt_EDA[j,i][[1]])/(1+g_bt[[Name]][j,i][[1]])) + NL_EDA_bt[[Name]][j,i][[1]]
      RY_EDA_bt[[Name]][j,i][[1]] <- ((1+r_ECB_bt[[Name]][j,i][[1]])/(1+g_bt[[Name]][j,i][[1]]))*RY_EDA_bt[[Name]][j-1,i][[1]]+
        + P_EDA_bt[[Name]][j,i][[1]]
      EL_EDA_bt[[Name]][j,i][[1]] <- L_EDA_bt[[Name]][j,i][[1]] + RY_EDA_bt[[Name]][j,i][[1]] - b_EDA_bt[[Name]][j,i][[1]]
      EN_EDA_bt[[Name]][j,i][[1]] <- EN_EDA_bt[[Name]][j-1,i][[1]]
      SC_EDA_bt[[Name]][j,i][[1]] <- L_EDA_bt[[Name]][j,i][[1]]+RY_EDA_bt[[Name]][j,i][[1]]+EN_EDA_bt[[Name]][j,i][[1]]+
        - b_EDA_bt[[Name]][j,i][[1]] -  EL_EDA_bt[[Name]][j,i][[1]]
      
      b_bt[[Name]][j,i][[1]] = b_bt[[Name]][j-1,i][[1]]* (1+ ((r_bt[[Name]][j,i][[1]] +
                                                                 - g_bt[[Name]][j,i][[1]])/(1+g_bt[[Name]][j,i][[1]]))) +
        - (1-phi[j,i][[1]])*PrSur_bt[[Name]][j,i][[1]]+
        - (1/m*(1/(1+g_bt[[Name]][j,i][[1]]))*bS_bt[[Name]][j-1,i][[1]] +
             + r_bt[[Name]][j,i][[1]]/(1+g_bt[[Name]][j,i][[1]])*bS_bt[[Name]][j-1,i][[1]])
      
      
      bS_bt[[Name]][j,i][[1]] = ((m-1)/m)*bS_bt[[Name]][j-1,i][[1]]*(1/(1+g_bt[[Name]][j,i][[1]]))
      
      bF_bt[[Name]][j,i][[1]] = b_bt[[Name]][j,i][[1]] - bS_bt[[Name]][j,i][[1]]
      
      LT_EDA_bt[[Name]][j,i][[1]] = 0.6*L_EDA_bt[[Name]][j,i][[1]]/(bF_bt[[Name]][j,i][[1]]+bS_bt[[Name]][j,i][[1]]+L_EDA_bt[[Name]][j,i][[1]])
      
      d_bt[[Name]][j,i][[1]] = b_bt[[Name]][j,i][[1]] + L_EDA_bt[[Name]][j,i][[1]]
      dT_bt[[Name]][j,i][[1]] = d_bt[[Name]][j,i][[1]] +
        - 10*beta*(bF_bt[[Name]][j,i][[1]]+
                     - b*(bF_bt[[Name]][j,i][[1]]/(bF_bt[[Name]][j,i][[1]]+bS_bt[[Name]][j,i][[1]]+L_EDA_bt[[Name]][j,i][[1]]))) +
        - 10*gamma*(bS_bt[[Name]][j,i][[1]]- b*(1- (bF_bt[[Name]][j,i][[1]]/(bF_bt[[Name]][j,i][[1]]+bS_bt[[Name]][j,i][[1]]+L_EDA_bt[[Name]][j,i][[1]]))))
      - 10*gamma*(L_EDA_bt[[Name]][j,i][[1]] - LT_EDA_bt[[Name]][j,i][[1]])
      
    } 
    m <-m-1
  }
  
  for (j in 7:21) {
    
    for (Name in MSList[MSList$Simulation=="Y",4])
    { g_bt[[Name]][j,i][[1]] <- Coeff_G[1, paste(Name)] + Coeff_G[2, paste(Name)]*IR1_bt_EDA[j-1,i][[1]] +
      + Coeff_G[3, paste(Name)]*(r_10Y_bt[[Name]][j-1,i][[1]]- r_10Y_bt[["DE"]][j-1,i][[1]]) +
      + Coeff_G[4, paste(Name)]*PrSur_bt[[Name]][j-1,i][[1]] + res1_bt[[Name]][j,i][[1]]  
    
    GDP[[Name]][j,i][[1]] <- (1+g_bt[[Name]][j,i][[1]])*GDP[[Name]][j-1,i][[1]] }
    
    
    AGG_GDP_EDA[j,i][[1]]<- GDP[["DE"]][j,i][[1]]+GDP[["ES"]][j,i][[1]]+GDP[["FR"]][j,i][[1]]+
      GDP[["GR"]][j,i][[1]]+GDP[["IT"]][j,i][[1]]+GDP[["PT"]][j,i][[1]]
    
    W[["DE"]][j,i][[1]]<- GDP[["DE"]][j,i][[1]]/AGG_GDP_EDA[j,i][[1]]
    W[["ES"]][j,i][[1]]<- GDP[["ES"]][j,i][[1]]/AGG_GDP_EDA[j,i][[1]]
    W[["FR"]][j,i][[1]]<- GDP[["FR"]][j,i][[1]]/AGG_GDP_EDA[j,i][[1]]
    W[["GR"]][j,i][[1]]<- GDP[["GR"]][j,i][[1]]/AGG_GDP_EDA[j,i][[1]]
    W[["IT"]][j,i][[1]]<- GDP[["IT"]][j,i][[1]]/AGG_GDP_EDA[j,i][[1]]
    W[["PT"]][j,i][[1]]<- GDP[["PT"]][j,i][[1]]/AGG_GDP_EDA[j,i][[1]]
    
    AGG_Gr_EDA[j,i][[1]] <- W[["DE"]][j-1,i][[1]]*g_bt[["DE"]][j,i][[1]] + W[["ES"]][j-1,i][[1]]*g_bt[["ES"]][j,i][[1]] +
      + W[["FR"]][j-1,i][[1]]*g_bt[["FR"]][j,i][[1]] + W[["GR"]][j-1,i][[1]]*g_bt[["GR"]][j,i][[1]] + 
      + W[["IT"]][j-1,i][[1]]*g_bt[["IT"]][j,i][[1]] + W[["PT"]][j-1,i][[1]]*g_bt[["PT"]][j,i][[1]]
    
    IR1_bt_EDA[j,i][[1]] <- b0 + b1*AGG_Gr_EDA[j,i][[1]] + b2*IR1_bt_EDA[j-1,i][[1]] + res2_bt[j,i]
    
    IR10_bt_EDA[j,i][[1]] <- c0 + c1*IR10_bt_EDA[j-1,i][[1]] + c2*IR1_bt_EDA[j,i][[1]] + res3_bt[j,i]
    
    r_ECB_bt[[Name]][j,i][[1]] <- g_bt[[Name]][j,i][[1]]
    
    r_10Y_bt[["DE"]][j,i][[1]] <- Coeff_DE[1,]+ Coeff_DE[1,]*r_10Y_bt[["DE"]][j-1,i][[1]] + 
      + Coeff_DE[3,]*IR1_bt[j,i][[1]] + res6_bt[j,i][[1]]
    
    r_bt[["DE"]][j,i][[1]] <- Coeff_CM[1,"DE"]*r_bt[["DE"]][j-1,i][[1]] + Coeff_CM[2, "DE"]*r_10Y_bt[["DE"]][j,i][[1]] +
      + Coeff_CM[3,"DE"]*IR1_bt[j,i][[1]] + res4_bt[["DE"]][j,i][[1]]
    
    for (Name in Names)
    { sigma[[Name]][j,i][[1]] = 0.004*b_bt[[Name]][j-1,i][[1]]
      r_10Y_bt[[Name]][j,i][[1]] <- r_10Y_bt[["DE"]][j,i][[1]] + Coeff_spread[1, paste(Name)] +
      + Coeff_spread[2, paste(Name)]*b_bt[[Name]][j-1,i][[1]] + res5_EDA_bt[[Name]][j,i][[1]]*sigma[[Name]][j,i][[1]] 
    
    r_bt[[Name]][j,i][[1]] <- Coeff_CM[1,paste(Name)]*r_bt[[Name]][j-1,i][[1]] + Coeff_CM[2, paste(Name)]*r_10Y_bt[[Name]][j,i][[1]] +
      + Coeff_CM[3,paste(Name)]*IR1_bt[j,i][[1]] +res4_bt[[Name]][j,i][[1]] 
    }
    
    for (Name in MSList[MSList$Simulation=="Y",4])
    {
      RF <- IR10_bt_EDA[j,i][[1]]
      alpha <- 1/(1+RF)
      B <- D_TTC/(1+RF)
      Bfirst <- unitVector-diag(B)
      Bfirst <- B/Bfirst 
      GrossPerpetuity <- alpha/(1-alpha)*unitVector
      GrossPerpetuity <- GrossPerpetuity-S_TTC%*%Bfirst%*%S_TTC_Inv%*%v  ### eq. 8 
      
      D_TTCInv <- solve(D_TTC)
      temp <- (UnitMatrix-D_TTCInv)%*%Bfirst
      temp <- rr*S_TTC%*%temp%*%S_TTC_Inv%*%v ### eq. 9
      temp <- (unitVector- temp)
      NetPerpetuity <- GrossPerpetuity/temp ### eq. 10
      cf <- 1/NetPerpetuity ### eq. 13 annual fundamental pricing
      
      MSCurrentRating <- Rating[, Name][[1]]
      PriceRecord <- Ratings[Ratings$Grade==MSCurrentRating,2]
      IC[[Name]][j,i][[1]] <- cf[PriceRecord]
    }
    
    for (Name in MSList[MSList$Simulation=="Y",4])
    {
      P_EDA_bt[[Name]][j,i][[1]]<- IC[[Name]][j,i][[1]]*(b_EDA_bt[[Name]][j-1,i][[1]] - RY_EDA_bt[[Name]][j-1,i][[1]])/(1+g_bt[[Name]][j,i][[1]])
      
      PrSur_bt[[Name]][j,i][[1]] = P_EDA_bt[[Name]][j,i][[1]] +
        + b_bt[[Name]][j-1, i][[1]]*((r_bt[[Name]][j,i][[1]]- g_bt[[Name]][j,i][[1]])/(1+g_bt[[Name]][j,i][[1]])) +
        - g_bt[[Name]][j,i][[1]]/(1+g_bt[[Name]][j,i][[1]])*L_EDA_bt[[Name]][j-1,i][[1]] +
        + delta*(d_bt[[Name]][j-1, i][[1]] -dT_bt[[Name]][j-1, i][[1]])
      
      
      NL_EDA_bt[[Name]][j,i][[1]] <- P_EDA_bt[[Name]][j,i][[1]] - phi[j,i][[1]]*(PrSur_bt[[Name]][j,i][[1]])
      
      L_EDA_bt[[Name]][j,i][[1]] <- (1/(1+g_bt[[Name]][j,i][[1]]))* L_EDA_bt[[Name]][j-1,i][[1]] + NL_EDA_bt[[Name]][j,i][[1]]
      b_EDA_bt[[Name]][j,i][[1]] <- b_EDA_bt[[Name]][j-1,i][[1]]*((1+IR10_bt_EDA[j,i][[1]])/(1+g_bt[[Name]][j,i][[1]])) + NL_EDA_bt[[Name]][j,i][[1]]
      RY_EDA_bt[[Name]][j,i][[1]] <- ((1+r_ECB_bt[[Name]][j,i][[1]])/(1+g_bt[[Name]][j,i][[1]]))*RY_EDA_bt[[Name]][j-1,i][[1]]+
        + P_EDA_bt[[Name]][j,i][[1]]
      EL_EDA_bt[[Name]][j,i][[1]] <- L_EDA_bt[[Name]][j,i][[1]] + RY_EDA_bt[[Name]][j,i][[1]] - b_EDA_bt[[Name]][j,i][[1]]
      EN_EDA_bt[[Name]][j,i][[1]] <- EN_EDA_bt[[Name]][j-1,i][[1]]
      SC_EDA_bt[[Name]][j,i][[1]] <- L_EDA_bt[[Name]][j,i][[1]]+RY_EDA_bt[[Name]][j,i][[1]]+EN_EDA_bt[[Name]][j,i][[1]]+
        - b_EDA_bt[[Name]][j,i][[1]] -  EL_EDA_bt[[Name]][j,i][[1]]
      
      b_bt[[Name]][j,i][[1]] = b_bt[[Name]][j-1, i][[1]]* (1+ ((r_bt[[Name]][j,i][[1]] +
                                                                  - g_bt[[Name]][j,i][[1]])/(1+g_bt[[Name]][j,i][[1]]))) +
        - (1-phi[j,i][[1]])*PrSur_bt[[Name]][j,i][[1]] 
      
      bS_bt[[Name]][j,i][[1]] = 0
      bF_bt[[Name]][j,i][[1]] = b_bt[[Name]][j,i][[1]] - bS_bt[[Name]][j,i][[1]]
      
      d_bt[[Name]][j,i][[1]] = b_bt[[Name]][j,i][[1]] + L_EDA_bt[[Name]][j,i][[1]]
      
      LT_EDA_bt[[Name]][j,i][[1]] = 0.6*L_EDA_bt[[Name]][j,i][[1]]/(bF_bt[[Name]][j,i][[1]]+bS_bt[[Name]][j,i][[1]]+L_EDA_bt[[Name]][j,i][[1]])
      
      d_bt[[Name]][j,i][[1]] = b_bt[[Name]][j,i][[1]] + L_EDA_bt[[Name]][j,i][[1]]
      dT_bt[[Name]][j,i][[1]] = d_bt[[Name]][j,i][[1]] +
        - 10*beta*(bF_bt[[Name]][j,i][[1]]+
                     - b*(bF_bt[[Name]][j,i][[1]]/(bF_bt[[Name]][j,i][[1]]+bS_bt[[Name]][j,i][[1]]+L_EDA_bt[[Name]][j,i][[1]]))) +
        - 10*gamma*(bS_bt[[Name]][j,i][[1]]- b*(1- (bF_bt[[Name]][j,i][[1]]/(bF_bt[[Name]][j,i][[1]]+bS_bt[[Name]][j,i][[1]]+L_EDA_bt[[Name]][j,i][[1]]))))
      - 10*gamma*(L_EDA_bt[[Name]][j,i][[1]] - LT_EDA_bt[[Name]][j,i][[1]])
    }
  }
}

for (Name in MSList[MSList$Simulation=="Y",4])
{ temp <- list(r_bt[[Name]], r_10Y_bt[[Name]], g_bt[[Name]],GDP[[Name]],W[[Name]], IC[[Name]],
               b_bt[[Name]], bS_bt[[Name]], bF_bt[[Name]],bT_bt[[Name]], d_bt[[Name]], dT_bt[[Name]],
               PrSur_bt[[Name]], P_EDA_bt[[Name]], NL_EDA_bt[[Name]], L_EDA_bt[[Name]], b_EDA_bt[[Name]],
               EL_EDA_bt[[Name]], RY_EDA_bt[[Name]], EN_EDA_bt[[Name]], SC_EDA_bt[[Name]], LT_EDA_bt[[Name]])
names(temp) <- c("r_bt", "r_10Y_bt", "g_bt","GDP","W", "IC",
                 "b_bt", "bS_bt", "bF_bt","bT_bt","d_bt", "dT_bt","PrSur_bt",
                 "P_EDA_bt", "NL_EDA_bt", "L_EDA_bt", "b_EDA_bt","EL_EDA_bt", "RY_EDA_bt", 
                 "EN_EDA_bt", "SC_EDA_bt", "LT_EDA_bt")
temp <- assign( paste("List_S1",Name, sep="_") , temp )}


rm(temp,r_bt, r_10Y_bt, g_bt,GDP,W, IC,b_bt, bS_bt,bF_bt,bT_bt,PrSur_bt,
   P_EDA_bt, NL_EDA_bt, L_EDA_bt, b_EDA_bt, EL_EDA_bt, RY_EDA_bt, EN_EDA_bt, SC_EDA_bt, LT_EDA_bt)

#-------------------------------------------------------------------------------
### EDA   
#-------------------------------------------------------------------------------
list_temp0<- list(List_S0_DE,List_S0_ES,List_S0_FR,List_S0_GR,List_S0_IT,List_S0_PT)
list_temp1<- list(List_S1_DE,List_S1_ES,List_S1_FR,List_S1_GR,List_S1_IT,List_S1_PT)
names(list_temp0)<- name
names(list_temp1)<- name

Total_GDP<- AGG_GDP_EDA

###scenario 1 
R_EDA_1 <- as.xts(array(0, c(21, nrep)), order.by=dates7)
L_EDA_1 <- as.xts(array(0, c(21, nrep)), order.by=dates7)
B_EDA_1 <- as.xts(array(0, c(21, nrep)), order.by=dates7)
SC_EDA_1 <- as.xts(array(0, c(21, nrep)), order.by=dates7)
EL_EDA_1 <- as.xts(array(0, c(21, nrep)), order.by=dates7)


test <- 0 
for (Name in MSList[MSList$Simulation=="Y",4])
{
  temp_R<- list_temp1[[Name]]$RY_EDA_bt
  temp_GDP <- list_temp1[[Name]]$GDP
  temp_B<- list_temp1[[Name]]$b_EDA_bt
  temp_L<- list_temp1[[Name]]$L_EDA_bt
  temp_EL<- list_temp1[[Name]]$EL_EDA_bt
  temp_SC<- list_temp1[[Name]]$SC_EDA_bt
  
  if (test==0) 
  {
    R_EDA_1 <- temp_R*temp_GDP/Total_GDP
    L_EDA_1 <- temp_L*temp_GDP/Total_GDP
    B_EDA_1 <- temp_B*temp_GDP/Total_GDP
    EL_EDA_1 <- temp_EL*temp_GDP/Total_GDP
    SC_EDA_1 <- temp_SC*temp_GDP/Total_GDP
    test <- 1
    
  } else {
    R_EDA_1 <- R_EDA_1 + temp_R*temp_GDP/Total_GDP
    L_EDA_1 <- L_EDA_1 + temp_L*temp_GDP/Total_GDP
    B_EDA_1 <- B_EDA_1 +temp_B*temp_GDP/Total_GDP
    EL_EDA_1 <-  EL_EDA_1 + temp_EL*temp_GDP/Total_GDP
    SC_EDA_1 <-  SC_EDA_1 + temp_SC*temp_GDP/Total_GDP
  } 
}


EDA_1 <- list(R_EDA_1, L_EDA_1, B_EDA_1,EL_EDA_1,SC_EDA_1)
names <- c("R_EDA_1", "L_EDA_1", "B_EDA_1","EL_EDA_1","SC_EDA_1")


temp_mean <- as.xts(array(NA, 21), order.by=dates7)
temp_q95 <- as.xts(array(NA, 21), order.by=dates7)

test<-0
for (k in 1:length(EDA_1))
{ temp<- EDA_1[[k]]
temp_mean <- as.xts(array(NA, 21), order.by=dates7)
temp_q95 <- as.xts(array(NA, 21), order.by=dates7)
for (i in 1:21){
  # obtaining the means and upper bound
  temp_mean[i] <- mean(temp[i, ])
  temp_q95[i] <- quantile(temp[i, ], 0.95)
}
if (test==0) 
{ tsdata<- cbind(temp_mean, temp_q95)
test <- 1

} else {
  tsdata<- cbind(tsdata, temp_mean, temp_q95)
} 
}

colnames(tsdata) <- c("R_EDA_1_mean", "R_EDA_1_q95","L_EDA_1_mean","L_EDA_1_q95",
                      "B_EDA_1_mean","B_EDA_1_q95","EL_EDA_1_mean","EL_EDA_1_q95",
                      "SC_EDA_1_mean", "SC_EDA_1_q95")

Stochastic_EDA <- tsdata


rm(R_EDA_1, L_EDA_1, B_EDA_1, EL_EDA_1, SC_EDA_1, tsdata, temp_mean, temp, temp_q95)
rm(temp_B,temp_GDP,temp_EL, temp_L,temp_R,temp_SC)


#-------------------------------------------------------------------------------
### DATA BACKUP  
#-------------------------------------------------------------------------------
IR10_bt_mean<- as.xts(array(NA, 21), order.by=dates7)
IR10_bt_q95<- as.xts(array(NA, 21), order.by=dates7)
IR1_bt_mean<- as.xts(array(NA, 21), order.by=dates7)
IR1_bt_q95<- as.xts(array(NA, 21), order.by=dates7)
IR10_bt_EDA_mean<- as.xts(array(NA, 21), order.by=dates7)
IR10_bt_EDA_q95<- as.xts(array(NA, 21), order.by=dates7)
IR1_bt_EDA_mean<- as.xts(array(NA, 21), order.by=dates7)
IR1_bt_EDA_q95<- as.xts(array(NA, 21), order.by=dates7)

for (i in 1:21){
  IR10_bt_mean[i] <- mean(IR10_bt[i, ])
  IR10_bt_q95[i] <- quantile(IR10_bt[i, ], 0.95)
  IR1_bt_mean[i] <- mean(IR1_bt[i, ])
  IR1_bt_q95[i] <- quantile(IR1_bt[i, ], 0.95)
  IR10_bt_EDA_mean[i] <- mean(IR10_bt_EDA[i, ])
  IR10_bt_EDA_q95[i] <- quantile(IR10_bt_EDA[i, ], 0.95)
  IR1_bt_EDA_mean[i] <- mean(IR1_bt_EDA[i, ])
  IR1_bt_EDA_q95[i] <- quantile(IR1_bt_EDA[i, ], 0.95)
}

Stochastic_EDA <- cbind(Stochastic_EDA, IR10_bt_mean, IR10_bt_q95, IR10_bt_EDA_mean, IR10_bt_EDA_q95,
                        IR1_bt_mean, IR1_bt_q95, IR1_bt_EDA_mean, IR1_bt_EDA_q95)
write.table(Stochastic_EDA, file = "data/EDA_2_scenarios_bund.csv", row.names = F, sep = ";", dec='.')

#-------### SCENARIO 0 ###-------

test =  0 
for (Name in MSList[MSList$Simulation=="Y",4])
{
  test1 <- 0 
  for (i in 1:11)
  { temp <- list_temp0[[Name]][[i]]
  temp_mean <- as.xts(array(NA, 21), order.by=dates7)
  temp_q95 <- as.xts(array(NA, 21), order.by=dates7)
  
  for (h in 1:21){
    temp_mean[h] <- mean(temp[h, ])
    temp_q95[h] <- quantile(temp[h, ], 0.95)
  }
  
  if (test1==0) 
  {   tsdata<- cbind(temp_mean, temp_q95)
  test1 <- 1
  
  } else {
    tsdata<- cbind(tsdata, temp_mean, temp_q95)
  } 
  }
  
  names = c("r_bt_mean","r_bt_q95", "r_10Y_bt_mean","r_10Y_bt_q95","g_bt_mean","g_bt_q95",
            "GDP_mean", "GDP_q95", "W_mean", "W_q95", "IC_mean", "IC_q95", 
            "b_bt_mean","b_bt_q95", "bS_bt_mean","bS_bt_q95","bF_bt_mean","bF_bt_q95",
            "bT_bt_mean","bT_bt_q95", "PrSur_bt_mean","PrSur_bt_q95")
  j=1
  for (i in names) {
    names[j] = paste(Name,i, sep="_")
    j=j+1
  }
  colnames(tsdata) = names 
  
  if (test==0) 
  {
    Total_Data_S0<- tsdata
    test <- 1
    
  } else {
    Total_Data_S0<- cbind(Total_Data_S0, tsdata)
  } 
}



#-------### SCENARIO 1 ###-------

test =  0 
for (Name in MSList[MSList$Simulation=="Y",4])
{
  test1 <- 0 
  for (i in 1:22)
  { temp <- list_temp1[[Name]][[i]]
  temp_mean <- as.xts(array(NA, 21), order.by=dates7)
  temp_q95 <- as.xts(array(NA, 21), order.by=dates7)
  
  for (h in 1:21){
    temp_mean[h] <- mean(temp[h, ])
    temp_q95[h] <- quantile(temp[h, ], 0.95)
  }
  
  if (test1==0) 
  {   tsdata<- cbind(temp_mean, temp_q95)
  test1 <- 1
  
  } else {
    tsdata<- cbind(tsdata, temp_mean, temp_q95)
  } 
  }
  
  names = c("r_bt_mean","r_bt_q95", "r_10Y_bt_mean","r_10Y_bt_q95","g_bt_mean","g_bt_q95",
            "GDP_mean", "GDP_q95", "W_mean", "W_q95", "IC_mean", "IC_q95", 
            "b_bt_mean","b_bt_q95", "bS_bt_mean","bS_bt_q95","bF_bt_mean","bF_bt_q95",
            "bT_bt_mean","bT_bt_q95", "d_bt_mean","d_bt_q95","dT_bt_mean","dT_bt_q95",
            "PrSur_bt_mean","PrSur_bt_q95","P_EDA_bt_mean", "P_EDA_bt_q95","NL_EDA_bt_mean","NL_EDA_bt_q95",
            "L_EDA_bt_mean","L_EDA_bt_q95","b_EDA_bt_mean","b_EDA_bt_q95","EL_EDA_bt_mean",
            "EL_EDA_bt_q95", "R_EDA_bt_mean","R_EDA_bt_q95","EN_EDA_bt_mean","EN_EDA_bt_q95",
            "SC_EDA_bt_mean","SC_EDA_bt_q95", "LT_EDA_bt_mean", "LT_EDA_bt_q95")
  j=1
  for (i in names) {
    names[j] = paste(Name,i, sep="_")
    j=j+1
  }
  colnames(tsdata) = names 
  
  if (test==0) 
  {
    Total_Data_S1<- tsdata
    test <- 1
    
  } else {
    Total_Data_S1<- cbind(Total_Data_S1, tsdata)
  } 
}

rm(temp, temp_mean, temp_q95,tsdata)

write.table(Total_Data_S0, file = "data/Data_Baseline_bund_sigma.csv", row.names = F, sep = ";", dec='.')
write.table(Total_Data_S1, file = "data/Data_EDA_scenario_bund_sigma.csv", row.names = F, sep = ";", dec='.')


